from typing import Any, Callable, Dict, List, Optional, Union

from pydantic import BaseModel

from cozepy import Chat, Message, ToolOutput
from cozepy.log import log_warning
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash
from cozepy.websockets.audio.transcriptions import (
    InputAudioBufferAppendEvent,
    InputAudioBufferCompletedEvent,
    InputAudioBufferCompleteEvent,
)
from cozepy.websockets.ws import (
    AsyncWebsocketsBaseClient,
    AsyncWebsocketsBaseEventHandler,
    InputAudio,
    OutputAudio,
    WebsocketsBaseClient,
    WebsocketsBaseEventHandler,
    WebsocketsEvent,
    WebsocketsEventType,
)


# req
class ChatUpdateEvent(WebsocketsEvent):
    class ChatConfig(BaseModel):
        conversation_id: Optional[str] = None
        user_id: Optional[str] = None
        meta_data: Optional[Dict[str, str]] = None
        custom_variables: Optional[Dict[str, str]] = None
        extra_params: Optional[Dict[str, str]] = None
        auto_save_history: Optional[bool] = None
        parameters: Optional[Dict[str, Any]] = None

    class Data(BaseModel):
        output_audio: Optional[OutputAudio] = None
        input_audio: Optional[InputAudio] = None
        chat_config: Optional["ChatUpdateEvent.ChatConfig"] = None

    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_UPDATE
    data: Data


# req
class ConversationChatSubmitToolOutputsEvent(WebsocketsEvent):
    class Data(BaseModel):
        chat_id: str
        tool_outputs: List[ToolOutput]

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_SUBMIT_TOOL_OUTPUTS
    data: Data


# req
class ConversationChatCancelEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CANCEL


# req
class ConversationMessageCreateEvent(WebsocketsEvent):
    class Data(BaseModel):
        role: str
        content_type: str
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_CREATE
    data: Data


# resp
class ChatCreatedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_CREATED


# resp
class ChatUpdatedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CHAT_UPDATED
    data: ChatUpdateEvent.Data


# resp
class ConversationChatCreatedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CREATED
    data: Chat


# resp
class ConversationChatInProgressEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_IN_PROGRESS


# resp
class ConversationMessageDeltaEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_DELTA
    data: Message


# resp
class ConversationAudioTranscriptCompletedEvent(WebsocketsEvent):
    class Data(BaseModel):
        content: str

    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED
    data: Data


# resp
class ConversationMessageCompletedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_MESSAGE_COMPLETED
    data: Message


# resp
class ConversationChatRequiresActionEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_REQUIRES_ACTION
    data: Chat


# resp
class ConversationAudioDeltaEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_DELTA
    data: Message


# resp
class ConversationAudioCompletedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_AUDIO_COMPLETED


# resp
class ConversationChatCompletedEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_COMPLETED
    data: Chat


# resp
class ConversationChatCanceledEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.CONVERSATION_CHAT_CANCELED


class WebsocketsChatEventHandler(WebsocketsBaseEventHandler):
    def on_chat_created(self, cli: "WebsocketsChatClient", event: ChatCreatedEvent):
        pass

    def on_chat_updated(self, cli: "WebsocketsChatClient", event: ChatUpdatedEvent):
        pass

    def on_input_audio_buffer_completed(self, cli: "WebsocketsChatClient", event: InputAudioBufferCompletedEvent):
        pass

    def on_conversation_chat_created(self, cli: "WebsocketsChatClient", event: ConversationChatCreatedEvent):
        pass

    def on_conversation_chat_in_progress(self, cli: "WebsocketsChatClient", event: ConversationChatInProgressEvent):
        pass

    def on_conversation_message_delta(self, cli: "WebsocketsChatClient", event: ConversationMessageDeltaEvent):
        pass

    def on_conversation_audio_transcript_completed(
        self, cli: "WebsocketsChatClient", event: ConversationAudioTranscriptCompletedEvent
    ):
        pass

    def on_conversation_message_completed(self, cli: "WebsocketsChatClient", event: ConversationMessageCompletedEvent):
        pass

    def on_conversation_chat_requires_action(
        self, cli: "WebsocketsChatClient", event: ConversationChatRequiresActionEvent
    ):
        pass

    def on_conversation_audio_delta(self, cli: "WebsocketsChatClient", event: ConversationAudioDeltaEvent):
        pass

    def on_conversation_audio_completed(self, cli: "WebsocketsChatClient", event: ConversationAudioCompletedEvent):
        pass

    def on_conversation_chat_completed(self, cli: "WebsocketsChatClient", event: ConversationChatCompletedEvent):
        pass

    def on_conversation_chat_canceled(self, cli: "WebsocketsChatClient", event: ConversationChatCanceledEvent):
        pass


class WebsocketsChatClient(WebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        bot_id: str,
        on_event: Union[WebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        **kwargs,
    ):
        if isinstance(on_event, WebsocketsChatEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/chat",
            query=remove_none_values(
                {
                    "bot_id": bot_id,
                    "workflow_id": workflow_id,
                }
            ),
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.CONVERSATION_CHAT_COMPLETED],
            **kwargs,
        )

    def chat_update(self, data: ChatUpdateEvent.Data) -> None:
        self._input_queue.put(ChatUpdateEvent.model_validate({"data": data}))

    def conversation_chat_submit_tool_outputs(self, data: ConversationChatSubmitToolOutputsEvent.Data) -> None:
        self._input_queue.put(ConversationChatSubmitToolOutputsEvent.model_validate({"data": data}))

    def conversation_chat_cancel(self) -> None:
        self._input_queue.put(ConversationChatCancelEvent.model_validate({}))

    def conversation_message_create(self, data: ConversationMessageCreateEvent.Data) -> None:
        self._input_queue.put(ConversationMessageCreateEvent.model_validate({"data": data}))

    def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    def input_audio_buffer_complete(self) -> None:
        self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    def _load_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        event_type = message.get("event_type") or ""
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.CHAT_CREATED.value:
            return ChatCreatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CHAT_UPDATED.value:
            return ChatUpdatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": ChatUpdateEvent.Data.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETED.value:
            return InputAudioBufferCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_CREATED.value:
            return ConversationChatCreatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_IN_PROGRESS.value:
            return ConversationChatInProgressEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_MESSAGE_DELTA.value:
            return ConversationMessageDeltaEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED.value:
            return ConversationAudioTranscriptCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": ConversationAudioTranscriptCompletedEvent.Data.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_REQUIRES_ACTION.value:
            return ConversationChatRequiresActionEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_MESSAGE_COMPLETED.value:
            return ConversationMessageCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_DELTA.value:
            return ConversationAudioDeltaEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_COMPLETED.value:
            return ConversationAudioCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_COMPLETED.value:
            return ConversationChatCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_CANCELED.value:
            return ConversationChatCanceledEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        else:
            log_warning("[%s] unknown event, type=%s, logid=%s", self._path, event_type, detail.logid)
        return None


class WebsocketsChatBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        bot_id: str,
        on_event: Union[WebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        **kwargs,
    ) -> WebsocketsChatClient:
        return WebsocketsChatClient(
            base_url=self._base_url,
            requester=self._requester,
            bot_id=bot_id,
            on_event=on_event,  # type: ignore
            workflow_id=workflow_id,
            **kwargs,
        )


class AsyncWebsocketsChatEventHandler(AsyncWebsocketsBaseEventHandler):
    async def on_chat_created(self, cli: "AsyncWebsocketsChatClient", event: ChatCreatedEvent):
        pass

    async def on_chat_updated(self, cli: "AsyncWebsocketsChatClient", event: ChatUpdatedEvent):
        pass

    async def on_input_audio_buffer_completed(
        self, cli: "AsyncWebsocketsChatClient", event: InputAudioBufferCompletedEvent
    ):
        pass

    async def on_conversation_chat_created(self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCreatedEvent):
        pass

    async def on_conversation_chat_in_progress(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatInProgressEvent
    ):
        pass

    async def on_conversation_message_delta(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationMessageDeltaEvent
    ):
        pass

    async def on_conversation_audio_transcript_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioTranscriptCompletedEvent
    ):
        pass

    async def on_conversation_chat_requires_action(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatRequiresActionEvent
    ):
        pass

    async def on_conversation_message_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationMessageCompletedEvent
    ):
        pass

    async def on_conversation_audio_delta(self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioDeltaEvent):
        pass

    async def on_conversation_audio_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationAudioCompletedEvent
    ):
        pass

    async def on_conversation_chat_completed(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCompletedEvent
    ):
        pass

    async def on_conversation_chat_canceled(
        self, cli: "AsyncWebsocketsChatClient", event: ConversationChatCanceledEvent
    ):
        pass


class AsyncWebsocketsChatClient(AsyncWebsocketsBaseClient):
    def __init__(
        self,
        base_url: str,
        requester: Requester,
        bot_id: str,
        on_event: Union[AsyncWebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        **kwargs,
    ):
        if isinstance(on_event, AsyncWebsocketsChatEventHandler):
            on_event = on_event.to_dict()
        super().__init__(
            base_url=base_url,
            requester=requester,
            path="v1/chat",
            query=remove_none_values(
                {
                    "bot_id": bot_id,
                    "workflow_id": workflow_id,
                }
            ),
            on_event=on_event,  # type: ignore
            wait_events=[WebsocketsEventType.CONVERSATION_CHAT_COMPLETED],
            **kwargs,
        )

    async def chat_update(self, data: ChatUpdateEvent.Data) -> None:
        await self._input_queue.put(ChatUpdateEvent.model_validate({"data": data}))

    async def conversation_chat_submit_tool_outputs(self, data: ConversationChatSubmitToolOutputsEvent.Data) -> None:
        await self._input_queue.put(ConversationChatSubmitToolOutputsEvent.model_validate({"data": data}))

    async def conversation_chat_cancel(self) -> None:
        await self._input_queue.put(ConversationChatCancelEvent.model_validate({}))

    async def conversation_message_create(self, data: ConversationMessageCreateEvent.Data) -> None:
        await self._input_queue.put(ConversationMessageCreateEvent.model_validate({"data": data}))

    async def input_audio_buffer_append(self, data: InputAudioBufferAppendEvent.Data) -> None:
        await self._input_queue.put(InputAudioBufferAppendEvent.model_validate({"data": data}))

    async def input_audio_buffer_complete(self) -> None:
        await self._input_queue.put(InputAudioBufferCompleteEvent.model_validate({}))

    def _load_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        event_type = message.get("event_type") or ""
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.CHAT_CREATED.value:
            return ChatCreatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CHAT_UPDATED.value:
            return ChatUpdatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": ChatUpdateEvent.Data.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.INPUT_AUDIO_BUFFER_COMPLETED.value:
            return InputAudioBufferCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_CREATED.value:
            return ConversationChatCreatedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_IN_PROGRESS.value:
            return ConversationChatInProgressEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_MESSAGE_DELTA.value:
            return ConversationMessageDeltaEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED.value:
            return ConversationAudioTranscriptCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": ConversationAudioTranscriptCompletedEvent.Data.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_REQUIRES_ACTION.value:
            return ConversationChatRequiresActionEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_MESSAGE_COMPLETED.value:
            return ConversationMessageCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_DELTA.value:
            return ConversationAudioDeltaEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Message.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_AUDIO_COMPLETED.value:
            return ConversationAudioCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_COMPLETED.value:
            return ConversationChatCompletedEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": Chat.model_validate(data),
                }
            )
        elif event_type == WebsocketsEventType.CONVERSATION_CHAT_CANCELED.value:
            return ConversationChatCanceledEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                }
            )
        else:
            log_warning("[%s] unknown event, type=%s, logid=%s", self._path, event_type, detail.logid)
        return None


class AsyncWebsocketsChatBuildClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        bot_id: str,
        on_event: Union[AsyncWebsocketsChatEventHandler, Dict[WebsocketsEventType, Callable]],
        workflow_id: Optional[str] = None,
        **kwargs,
    ) -> AsyncWebsocketsChatClient:
        return AsyncWebsocketsChatClient(
            base_url=self._base_url,
            requester=self._requester,
            bot_id=bot_id,
            on_event=on_event,  # type: ignore
            workflow_id=workflow_id,
            **kwargs,
        )



import abc
from typing import (
    TYPE_CHECKING,
    Any,
    AsyncIterator,
    Callable,
    Coroutine,
    Dict,
    Generic,
    Iterable,
    Iterator,
    List,
    Optional,
    Tuple,
    TypeVar,
    Union,
    cast,
    overload,
)

import httpx
from pydantic import BaseModel, ConfigDict
from typing_extensions import SupportsIndex

from cozepy.exception import CozeInvalidEventError
from cozepy.log import log_debug

if TYPE_CHECKING:
    from cozepy.request import Requester

T = TypeVar("T")
SyncPage = TypeVar("SyncPage", bound="PagedBase")
AsyncPage = TypeVar("AsyncPage", bound="AsyncPagedBase")


class HTTPResponse(object):
    def __init__(self, raw_response: httpx.Response):
        self._raw_response = raw_response

    @property
    def logid(self) -> str:
        if self._raw_response is None:
            return ""
        return self._raw_response.headers.get("x-tt-logid")


class CozeModel(BaseModel):
    model_config = ConfigDict(
        protected_namespaces=(),
        arbitrary_types_allowed=True,
    )
    _raw_response: Optional[httpx.Response] = None

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)  # type: ignore


class IteratorHTTPResponse(HTTPResponse, Generic[T]):
    def __init__(self, raw_response: httpx.Response, data: Iterator[T]):
        super().__init__(raw_response)
        self.data = data


class AsyncIteratorHTTPResponse(HTTPResponse, Generic[T]):
    def __init__(self, raw_response: httpx.Response, data: AsyncIterator[T]):
        super().__init__(raw_response)
        self.data = data


class FileHTTPResponse(object):
    def __init__(self, raw_response: httpx.Response):
        self._raw_response = raw_response

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)

    def write_to_file(self, file: Union[str]) -> None:
        with open(file, mode="wb") as f:
            for data in self._raw_response.iter_bytes():
                f.write(data)


class ListResponse(Generic[T]):
    def __init__(self, raw_response: httpx.Response, data: List[T]):
        self.data = data
        self._raw_response = raw_response

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)

    def __len__(self) -> int:
        return len(self.data)

    def __iter__(self) -> Iterator[T]:
        return iter(self.data)

    @overload
    def __getitem__(self, key: SupportsIndex) -> T: ...

    @overload
    def __getitem__(self, key: slice) -> List[T]: ...

    def __getitem__(self, key: Union[SupportsIndex, slice]) -> Union[T, List[T]]:
        return self.data[key]

    @overload
    def __setitem__(self, key: SupportsIndex, value: T) -> None: ...

    @overload
    def __setitem__(self, key: slice, value: Iterable[T]) -> None: ...

    def __setitem__(self, key: Union[SupportsIndex, slice], value: Union[T, Iterable[T]]) -> None:
        if isinstance(key, slice):
            if not isinstance(value, Iterable):
                raise TypeError("Can only assign an iterable to slice")
            self.data[key] = value  # type: ignore
        else:
            if isinstance(value, Iterable):
                raise TypeError("Can only assign a single value to index")
            self.data[key] = value

    def __delitem__(self, key: Union[SupportsIndex, slice]) -> None:
        del self.data[key]

    def __contains__(self, key: object) -> bool:
        return key in self.data

    def __reversed__(self) -> Iterator[T]:
        return reversed(self.data)


class HTTPRequest(CozeModel, Generic[T]):
    method: str
    url: str
    params: Optional[dict] = None
    headers: Optional[dict] = None
    json_body: Optional[dict] = None
    files: Optional[dict] = None
    is_async: Optional[bool] = None
    stream: bool = False
    data_field: str = "data"
    cast: Optional[Any] = None

    @property
    def as_httpx(self) -> httpx.Request:
        if self.files and self.json_body:
            return httpx.Request(
                method=self.method,
                url=self.url,
                params=self.params,
                headers=self.headers,
                data=self.json_body,
                files=self.files,
            )
        return httpx.Request(
            method=self.method,
            url=self.url,
            params=self.params,
            headers=self.headers,
            json=self.json_body,
            files=self.files,
        )


class PagedBase(Generic[T], abc.ABC):
    @abc.abstractmethod
    def iter_pages(self: SyncPage) -> Iterator[SyncPage]:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def items(self) -> List[T]:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def has_more(self) -> bool:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def response(self) -> HTTPResponse:
        raise NotImplementedError


class AsyncPagedBase(Generic[T], abc.ABC):
    @abc.abstractmethod
    def iter_pages(self: AsyncPage) -> AsyncIterator[AsyncPage]:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def items(self) -> List[T]:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def has_more(self) -> bool:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def response(self) -> HTTPResponse:
        raise NotImplementedError


class NumberPagedResponse(Generic[T], abc.ABC):
    @abc.abstractmethod
    def get_total(self) -> Optional[int]:
        raise NotImplementedError

    @abc.abstractmethod
    def get_has_more(self) -> Optional[bool]:
        raise NotImplementedError

    @abc.abstractmethod
    def get_items(self) -> List[T]:
        raise NotImplementedError

    @property
    @abc.abstractmethod
    def response(self) -> HTTPResponse:
        raise NotImplementedError


class NumberPaged(PagedBase[T]):
    def __init__(
        self,
        page_num: int,
        page_size: int,
        requestor: "Requester",
        request_maker: Callable[[int, int], HTTPRequest],
    ):
        self.page_num = page_num
        self.page_size = page_size

        self._total = None
        self._has_more = None
        self._items = None
        self._http_response: Optional[HTTPResponse] = None

        self._requestor = requestor
        self._request_maker = request_maker

        self._fetch_page()

    def __iter__(self) -> Iterator[T]:  # type: ignore
        for page in self.iter_pages():
            for item in page.items:
                yield item

    def iter_pages(self) -> Iterator["NumberPaged[T]"]:
        yield self

        page_num = self.page_num
        current_page = self
        while NumberPaged._is_page_has_more(current_page):
            page_num += 1
            current_page = NumberPaged(
                page_num=page_num,
                page_size=self.page_size,
                requestor=self._requestor,
                request_maker=self._request_maker,
            )
            yield current_page

    @property
    def response(self) -> HTTPResponse:
        return self._http_response  # type: ignore

    @property
    def items(self) -> List[T]:
        return self._items or cast(List[T], [])

    @property
    def has_more(self) -> bool:
        return NumberPaged._is_page_has_more(self)

    @property
    def total(self) -> int:
        return self._total or 0

    def _fetch_page(self):
        if self._total is not None or self._has_more is not None:
            return
        request: HTTPRequest = self._request_maker(self.page_num, self.page_size)
        res: NumberPagedResponse[T] = self._requestor.send(request)
        self._total = res.get_total()
        self._has_more = res.get_has_more()
        self._items = res.get_items()
        self._http_response = res.response

    @staticmethod
    def _is_page_has_more(page: "NumberPaged[T]") -> bool:
        if page._has_more is not None:
            return page._has_more
        if page._total is not None:
            return page._total > page.page_num * page.page_size
        if page._items is not None and page._items and len(page._items) >= page.page_size:
            return True

        return False


class AsyncNumberPaged(AsyncPagedBase[T]):
    def __init__(
        self,
        page_num: int,
        page_size: int,
        requestor: "Requester",
        request_maker: Callable[[int, int], Coroutine[None, None, HTTPRequest]],
    ):
        self.page_num = page_num
        self.page_size = page_size

        self._total = None
        self._has_more = None
        self._items = None
        self._raw_response: Optional[httpx.Response] = None

        self._requestor = requestor
        self._request_maker = request_maker

    async def __aiter__(self) -> AsyncIterator[T]:
        async for page in self.iter_pages():
            for item in page.items:
                yield item

    async def iter_pages(self) -> AsyncIterator["AsyncNumberPaged[T]"]:
        yield self

        page_num = self.page_num
        current_page = self
        while AsyncNumberPaged._is_page_has_more(current_page):
            page_num += 1
            page: AsyncNumberPaged[T] = await AsyncNumberPaged.build(
                page_num=page_num,
                page_size=self.page_size,
                requestor=self._requestor,
                request_maker=self._request_maker,
            )
            current_page = page
            yield page

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)  # type: ignore

    @property
    def items(self) -> List[T]:
        return self._items or cast(List[T], [])

    @property
    def has_more(self) -> bool:
        return AsyncNumberPaged._is_page_has_more(self)

    @property
    def total(self) -> int:
        return self._total or 0

    async def _fetch_page(self):
        """

        :rtype: object
        """
        if self._total is not None:
            return
        request = await self._request_maker(self.page_num, self.page_size)
        res: NumberPagedResponse[T] = await self._requestor.asend(request)
        self._total = res.get_total()
        self._has_more = res.get_has_more()
        self._items = res.get_items()
        self._raw_response = res._raw_response

    @staticmethod
    def _is_page_has_more(page: "AsyncNumberPaged[T]") -> bool:
        if page._has_more is not None:
            return page._has_more
        if page._total is not None:
            return page._total > page.page_num * page.page_size
        if page._total is not None and page._items and len(page._items) >= page.page_size:
            return True
        return False

    @staticmethod
    async def build(
        page_num: int,
        page_size: int,
        requestor: "Requester",
        request_maker: Callable[[int, int], Coroutine[None, None, HTTPRequest]],
    ) -> "AsyncNumberPaged[T]":
        page: AsyncNumberPaged[T] = AsyncNumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=requestor,
            request_maker=request_maker,
        )
        await page._fetch_page()
        return page


class LastIDPagedResponse(Generic[T], abc.ABC):
    @abc.abstractmethod
    def get_first_id(self) -> str: ...

    @abc.abstractmethod
    def get_last_id(self) -> str: ...

    @abc.abstractmethod
    def get_has_more(self) -> bool: ...

    @abc.abstractmethod
    def get_items(self) -> List[T]: ...


class LastIDPaged(PagedBase[T]):
    def __init__(
        self,
        before_id: str,
        after_id: str,
        requestor: "Requester",
        request_maker: Callable[[str, str], HTTPRequest],
    ):
        self.before_id = before_id
        self.after_id = after_id
        self.first_id: Optional[str] = None
        self.last_id: Optional[str] = None
        self._has_more: Optional[bool] = None
        self._items: Optional[List[T]] = None
        self._raw_response: Optional[httpx.Response] = None

        self._requestor = requestor
        self._request_maker = request_maker

        self._fetch_page()

    def __iter__(self) -> Iterator[T]:  # type: ignore
        for page in self.iter_pages():
            for item in page.items:
                yield item

    def iter_pages(self) -> Iterator["LastIDPaged[T]"]:
        yield self

        has_more = self._has_more
        last_id = self.last_id
        while self._check_has_more(has_more, last_id):
            page: LastIDPaged[T] = LastIDPaged(
                before_id="",
                after_id=last_id or "",
                requestor=self._requestor,
                request_maker=self._request_maker,
            )
            has_more = page.has_more
            last_id = page.last_id
            yield page

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)  # type: ignore

    @property
    def items(self) -> List[T]:
        return self._items or []

    @property
    def has_more(self) -> bool:
        if self._has_more is not None:
            return self._has_more
        return self.after_id != ""

    def _fetch_page(self):
        if self.last_id is not None or self._has_more is not None:
            return

        request = self._request_maker(self.before_id, self.after_id)
        res: LastIDPagedResponse[T] = self._requestor.send(request)

        self.first_id = res.get_first_id()
        self.last_id = res.get_last_id()
        self._has_more = res.get_has_more()
        self._items = res.get_items()
        self._raw_response = res._raw_response

    def _check_has_more(self, has_more: Optional[bool] = None, last_id: Optional[str] = None) -> bool:
        if has_more is not None:
            return has_more
        if last_id and last_id != "":
            return True
        return False


class AsyncLastIDPaged(AsyncPagedBase[T]):
    def __init__(
        self,
        before_id: str,
        after_id: str,
        requestor: "Requester",
        request_maker: Callable[[str, str], Coroutine[None, None, HTTPRequest]],
    ):
        self.before_id = before_id
        self.after_id = after_id
        self.first_id = None
        self.last_id = None
        self._has_more = None
        self._items: List[T] = []
        self._raw_response: Optional[httpx.Response] = None

        self._requestor = requestor
        self._request_maker = request_maker

    async def __aiter__(self) -> AsyncIterator[T]:
        async for page in self.iter_pages():
            for item in page.items:
                yield item

    async def iter_pages(self) -> AsyncIterator["AsyncLastIDPaged[T]"]:
        yield self

        has_more = self._has_more
        last_id = self.last_id
        while self._check_has_more(has_more, last_id):
            page: AsyncLastIDPaged[T] = await AsyncLastIDPaged.build(
                before_id="",
                after_id=last_id or "",
                requestor=self._requestor,
                request_maker=self._request_maker,
            )
            has_more = page.has_more
            last_id = page.last_id
            yield page

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)  # type: ignore

    @property
    def items(self) -> List[T]:
        return self._items

    @property
    def has_more(self) -> bool:
        if self._has_more is not None:
            return self._has_more
        return self.after_id != ""

    @staticmethod
    async def build(
        before_id: str,
        after_id: str,
        requestor: "Requester",
        request_maker: Callable[[str, str], Coroutine[None, None, HTTPRequest]],
    ) -> "AsyncLastIDPaged[T]":
        page: AsyncLastIDPaged = AsyncLastIDPaged(
            before_id=before_id,
            after_id=after_id,
            requestor=requestor,
            request_maker=request_maker,
        )
        await page._fetch_page()
        return page

    async def _fetch_page(self):
        if self.last_id is not None or self._has_more is not None:
            return

        request = await self._request_maker(self.before_id, self.after_id)
        res: LastIDPagedResponse[T] = await self._requestor.asend(request)

        self.first_id = res.get_first_id()
        self.last_id = res.get_last_id()
        self._has_more = res.get_has_more()
        self._items = res.get_items()
        self._raw_response = res._raw_response

    def _check_has_more(self, has_more: Optional[bool] = None, last_id: Optional[str] = None) -> bool:
        if has_more is not None:
            return has_more
        if last_id and last_id != "":
            return True
        return False


class Stream(Generic[T]):
    def __init__(
        self,
        raw_response: httpx.Response,
        iters: Iterator[str],
        fields: List[str],
        handler: Callable[[Dict[str, str], httpx.Response], T],
    ):
        self._iters = iters
        self._fields = fields
        self._handler = handler
        self._raw_response = raw_response

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)

    def __iter__(self):
        return self

    def __next__(self) -> T:
        return self._handler(self._extra_event(), self._raw_response)

    def _extra_event(self) -> Dict[str, str]:
        data = dict(map(lambda x: (x, ""), self._fields))
        times = 0

        while times < len(data):
            line = next(self._iters).strip()
            if line == "":
                continue

            log_debug("receive event, logid=%s, event=%s", self.response.logid, line)

            field, value = self._extra_field_data(line, data)
            data[field] = value
            times += 1
        return data

    def _extra_field_data(self, line: str, data: Dict[str, str]) -> Tuple[str, str]:
        for field in self._fields:
            if line.startswith(field + ":"):
                if data[field] == "":
                    return field, line[len(field) + 1 :].strip()
                else:
                    raise CozeInvalidEventError(field, line, self.response.logid)
        raise CozeInvalidEventError("", line, self.response.logid)


class AsyncStream(Generic[T]):
    def __init__(
        self,
        iters: AsyncIterator[str],
        fields: List[str],
        handler: Callable[[Dict[str, str], httpx.Response], T],
        raw_response: httpx.Response,
    ):
        self._iters = iters
        self._fields = fields
        self._handler = handler
        self._iterator = self.__stream__()
        self._raw_response = raw_response

    @property
    def response(self) -> HTTPResponse:
        return HTTPResponse(self._raw_response)

    async def __aiter__(self) -> AsyncIterator[T]:
        async for item in self._iterator:
            yield item

    async def __anext__(self) -> T:
        return await self._iterator.__anext__()

    async def __stream__(self) -> AsyncIterator[T]:
        data = self._make_data()
        times = 0

        async for line in self._iters:
            line = line.strip()
            if line == "":
                continue

            log_debug("async receive event, logid=%s, event=%s", self.response.logid, line)

            field, value = self._extra_field_data(line, data)
            data[field] = value
            times += 1

            if times >= len(self._fields):
                try:
                    yield self._handler(data, self._raw_response)
                except StopAsyncIteration:
                    return
                data = self._make_data()
                times = 0

    def _extra_field_data(self, line: str, data: Dict[str, str]) -> Tuple[str, str]:
        for field in self._fields:
            if line.startswith(field + ":"):
                if data[field] == "":
                    return field, line[len(field) + 1 :].strip()
                else:
                    raise CozeInvalidEventError(field, line, self.response.logid)
        raise CozeInvalidEventError("", line, self.response.logid)

    def _make_data(self):
        return dict(map(lambda x: (x, ""), self._fields))



from enum import Enum
from typing import Optional

from cozepy.model import FileHTTPResponse
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class AudioFormat(str, Enum):
    WAV = "wav"
    PCM = "pcm"
    OGG_OPUS = "ogg_opus"
    MP3 = "mp3"


class LanguageCode(str, Enum):
    ZH = "zh"
    EN = "en"
    JA = "ja"
    ES = "es"
    ID = "id"
    PT = "pt"


class SpeechClient(object):
    """
    speech service client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        input: str,
        voice_id: str,
        response_format: AudioFormat = AudioFormat.MP3,
        speed: float = 1,
        sample_rate: int = 24000,
        **kwargs,
    ) -> FileHTTPResponse:
        """
        Generate speech audio from input text with specified voice

        :param input: The text to generate audio for. Maximum length is 1024 characters.
        :param voice_id: The voice ID to generate audio with, obtained via .audio.voices.list
        :param response_format: Audio encoding format, wav / pcm / ogg_opus / mp3, defaults to mp3
        :param speed: Speech speed, [0.2,3], defaults to 1, typically one decimal place is sufficient
        :param sample_rate: Audio sample rate, defaults to 24000, available values: [8000, 16000, 22050, 24000, 32000, 44100, 48000]
        :return: The synthesized audio file content
        """
        url = f"{self._base_url}/v1/audio/speech"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "input": input,
            "voice_id": voice_id,
            "response_format": response_format,
            "speed": speed,
            "sample_rate": sample_rate,
        }
        return self._requester.request("post", url, stream=False, cast=FileHTTPResponse, headers=headers, body=body)


class AsyncSpeechClient(object):
    """
    speech service client.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def create(
        self,
        *,
        input: str,
        voice_id: str,
        response_format: AudioFormat = AudioFormat.MP3,
        speed: float = 1,
        sample_rate: int = 24000,
        **kwargs,
    ) -> FileHTTPResponse:
        """
        Generate speech audio from input text with specified voice

        :param input: The text to generate audio for. Maximum length is 1024 characters.
        :param voice_id: The voice ID to generate audio with, obtained via .audio.voices.list
        :param response_format: Audio encoding format, wav / pcm / ogg_opus / mp3, defaults to mp3
        :param speed: Speech speed, [0.2,3], defaults to 1, typically one decimal place is sufficient
        :param sample_rate: Audio sample rate, defaults to 24000, available values: [8000, 16000, 22050, 24000, 32000, 44100, 48000]
        :return: The synthesized audio file content
        """
        url = f"{self._base_url}/v1/audio/speech"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "input": input,
            "voice_id": voice_id,
            "response_format": response_format,
            "speed": speed,
            "sample_rate": sample_rate,
        }
        return await self._requester.arequest(
            "post", url, stream=False, cast=FileHTTPResponse, headers=headers, body=body
        )



from typing import TYPE_CHECKING, Optional

from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash

if TYPE_CHECKING:
    from .chat import AsyncWorkflowsChatClient, WorkflowsChatClient
    from .runs import AsyncWorkflowsRunsClient, WorkflowsRunsClient


class WorkflowsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._runs: Optional[WorkflowsRunsClient] = None
        self._chat: Optional[WorkflowsChatClient] = None

    @property
    def runs(self) -> "WorkflowsRunsClient":
        if not self._runs:
            from .runs import WorkflowsRunsClient

            self._runs = WorkflowsRunsClient(self._base_url, self._requester)
        return self._runs

    @property
    def chat(self) -> "WorkflowsChatClient":
        if not self._chat:
            from .chat import WorkflowsChatClient

            self._chat = WorkflowsChatClient(self._base_url, self._requester)
        return self._chat


class AsyncWorkflowsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._runs: Optional[AsyncWorkflowsRunsClient] = None
        self._chat: Optional[AsyncWorkflowsChatClient] = None

    @property
    def runs(self) -> "AsyncWorkflowsRunsClient":
        if not self._runs:
            from .runs import AsyncWorkflowsRunsClient

            self._runs = AsyncWorkflowsRunsClient(self._base_url, self._requester)
        return self._runs

    @property
    def chat(self) -> "AsyncWorkflowsChatClient":
        if not self._chat:
            from .chat import AsyncWorkflowsChatClient

            self._chat = AsyncWorkflowsChatClient(self._base_url, self._requester)
        return self._chat



import warnings
from typing import TYPE_CHECKING, Optional

from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash

if TYPE_CHECKING:
    from .documents import AsyncDocumentsClient, DocumentsClient


class KnowledgeClient(object):
    def __init__(self, base_url: str, requester: Requester):
        warnings.warn(
            "The 'coze.knowledge' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._documents: Optional[DocumentsClient] = None

    @property
    def documents(self) -> "DocumentsClient":
        warnings.warn(
            "The 'coze.knowledge.documents' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        if self._documents is None:
            from .documents import DocumentsClient

            self._documents = DocumentsClient(base_url=self._base_url, requester=self._requester)
        return self._documents


class AsyncKnowledgeClient(object):
    def __init__(self, base_url: str, requester: Requester):
        warnings.warn(
            "The 'coze.knowledge' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._documents: Optional[AsyncDocumentsClient] = None

    @property
    def documents(self) -> "AsyncDocumentsClient":
        warnings.warn(
            "The 'coze.knowledge.documents' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        if self._documents is None:
            from .documents import AsyncDocumentsClient

            self._documents = AsyncDocumentsClient(base_url=self._base_url, requester=self._requester)
        return self._documents



import warnings
from typing import List, Optional

from cozepy.datasets.documents import (
    Document,
    DocumentBase,
    DocumentChunkStrategy,
    DocumentUpdateRule,
)
from cozepy.model import AsyncNumberPaged, CozeModel, HTTPRequest, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class _PrivateListDocumentsData(CozeModel, NumberPagedResponse[Document]):
    document_infos: List[Document]
    total: int

    def get_total(self) -> Optional[int]:
        return self.total

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[Document]:
        return self.document_infos


class DocumentsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        warnings.warn(
            "The 'coze.knowledge.documents' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def create(
        self,
        *,
        dataset_id: str,
        document_bases: List[DocumentBase],
        chunk_strategy: Optional[DocumentChunkStrategy] = None,
    ) -> List[Document]:
        warnings.warn(
            "The 'coze.knowledge.documents.create' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.create' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Upload files to the specific knowledge.

        docs en: https://www.coze.com/docs/developer_guides/create_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/create_knowledge_files

        :param dataset_id: The ID of the knowledge base.
        :param document_bases: The metadata information of the files awaiting upload. The array has a maximum length of
        10, meaning up to 10 files can be uploaded at a time. For detailed instructions, refer to the DocumentBase
        object.
        :param chunk_strategy: Chunk strategy. These rules must be set only when uploading a file to a new knowledge
        for the first time. For subsequent file uploads to this knowledge, it is not necessary to pass these rules; the
        default is to continue using the initial settings, and modifications are not supported.
        For detailed instructions, refer to the ChunkStrategy object.
        :return: list of Document
        """
        url = f"{self._base_url}/open_api/knowledge/document/create"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "dataset_id": dataset_id,
            "document_bases": [i.model_dump() for i in document_bases],
            "chunk_strategy": chunk_strategy.model_dump() if chunk_strategy else None,
        }
        return self._requester.request(
            "post", url, False, [Document], headers=headers, body=body, data_field="document_infos"
        )

    def update(
        self,
        *,
        document_id: str,
        document_name: Optional[str] = None,
        update_rule: Optional[DocumentUpdateRule] = None,
    ) -> None:
        warnings.warn(
            "The 'coze.knowledge.documents.update' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.update' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Modify the knowledge base file name and update strategy.

        docs en: https://www.coze.com/docs/developer_guides/modify_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/modify_knowledge_files

        :param document_id: The ID of the knowledge base file.
        :param document_name: The new name of the knowledge base file.
        :param update_rule: The update strategy for online web pages. Defaults to no automatic updates.
        For detailed information, refer to the UpdateRule object.
        :return: None
        """
        url = f"{self._base_url}/open_api/knowledge/document/update"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "document_id": document_id,
            "document_name": document_name,
            "update_rule": update_rule,
        }
        return self._requester.request(
            "post",
            url,
            False,
            None,
            headers=headers,
            body=body,
        )

    def delete(
        self,
        *,
        document_ids: List[str],
    ) -> None:
        warnings.warn(
            "The 'coze.knowledge.documents.delete' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.delete' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Delete text, images, sheets, and other files in the knowledge base, supporting batch deletion.

        docs en: https://www.coze.com/docs/developer_guides/delete_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/delete_knowledge_files

        :param document_ids: The list of knowledge base files to be deleted. The maximum length of the array is 100,
        meaning a maximum of 100 files can be deleted at one time.
        :return: None
        """
        url = f"{self._base_url}/open_api/knowledge/document/delete"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "document_ids": document_ids,
        }
        return self._requester.request(
            "post",
            url,
            False,
            None,
            headers=headers,
            body=body,
        )

    def list(
        self,
        *,
        dataset_id: str,
        page_num: int = 1,
        page_size: int = 10,
    ) -> NumberPaged[Document]:
        warnings.warn(
            "The 'coze.knowledge.documents.list' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.list' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        View the file list of a specified knowledge base, which includes lists of documents, spreadsheets, or images.

        docs en: https://www.coze.com/docs/developer_guides/list_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/list_knowledge_files


        :param dataset_id: The ID of the knowledge base.
        :param page_num: The page number for paginated queries. Default is 1, meaning the data return starts from the
        first page.
        :param page_size: The size of pagination. Default is 10, meaning that 10 data entries are returned per page.
        :return: list of Document
        """
        url = f"{self._base_url}/open_api/knowledge/document/list"
        headers = {"Agw-Js-Conv": "str"}

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "POST",
                url,
                headers=headers,
                json={
                    "dataset_id": dataset_id,
                    "page": i_page_num,
                    "size": i_page_size,
                },
                cast=_PrivateListDocumentsData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncDocumentsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        warnings.warn(
            "The 'coze.knowledge.documents' module is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def create(
        self,
        *,
        dataset_id: str,
        document_bases: List[DocumentBase],
        chunk_strategy: Optional[DocumentChunkStrategy] = None,
    ) -> List[Document]:
        warnings.warn(
            "The 'coze.knowledge.documents.create' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.create' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Upload files to the specific knowledge.

        docs en: https://www.coze.com/docs/developer_guides/create_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/create_knowledge_files

        :param dataset_id: The ID of the knowledge base.
        :param document_bases: The metadata information of the files awaiting upload. The array has a maximum length of
        10, meaning up to 10 files can be uploaded at a time. For detailed instructions, refer to the DocumentBase
        object.
        :param chunk_strategy: Chunk strategy. These rules must be set only when uploading a file to a new knowledge
        for the first time. For subsequent file uploads to this knowledge, it is not necessary to pass these rules; the
        default is to continue using the initial settings, and modifications are not supported.
        For detailed instructions, refer to the ChunkStrategy object.
        :return: list of Document
        """
        url = f"{self._base_url}/open_api/knowledge/document/create"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "dataset_id": dataset_id,
            "document_bases": [i.model_dump() for i in document_bases],
            "chunk_strategy": chunk_strategy.model_dump() if chunk_strategy else None,
        }
        return await self._requester.arequest(
            "post", url, False, [Document], headers=headers, body=body, data_field="document_infos"
        )

    async def update(
        self,
        *,
        document_id: str,
        document_name: Optional[str] = None,
        update_rule: Optional[DocumentUpdateRule] = None,
    ) -> None:
        warnings.warn(
            "The 'coze.knowledge.documents.update' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.update' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Modify the knowledge base file name and update strategy.

        docs en: https://www.coze.com/docs/developer_guides/modify_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/modify_knowledge_files

        :param document_id: The ID of the knowledge base file.
        :param document_name: The new name of the knowledge base file.
        :param update_rule: The update strategy for online web pages. Defaults to no automatic updates.
        For detailed information, refer to the UpdateRule object.
        :return: None
        """
        url = f"{self._base_url}/open_api/knowledge/document/update"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "document_id": document_id,
            "document_name": document_name,
            "update_rule": update_rule,
        }
        return await self._requester.arequest(
            "post",
            url,
            False,
            cast=None,
            headers=headers,
            body=body,
        )

    async def delete(
        self,
        *,
        document_ids: List[str],
    ) -> None:
        warnings.warn(
            "The 'coze.knowledge.documents.delete' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.delete' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        Delete text, images, sheets, and other files in the knowledge base, supporting batch deletion.

        docs en: https://www.coze.com/docs/developer_guides/delete_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/delete_knowledge_files

        :param document_ids: The list of knowledge base files to be deleted. The maximum length of the array is 100,
        meaning a maximum of 100 files can be deleted at one time.
        :return: None
        """
        url = f"{self._base_url}/open_api/knowledge/document/delete"
        headers = {"Agw-Js-Conv": "str"}
        body = {
            "document_ids": document_ids,
        }
        return await self._requester.arequest(
            "post",
            url,
            False,
            cast=None,
            headers=headers,
            body=body,
        )

    async def list(
        self,
        *,
        dataset_id: str,
        page_num: int = 1,
        page_size: int = 10,
    ) -> AsyncNumberPaged[Document]:
        warnings.warn(
            "The 'coze.knowledge.documents.list' method is deprecated and will be removed in a future version. "
            "Please use 'coze.datasets.documents.list' instead.",
            DeprecationWarning,
            stacklevel=2,
        )
        """
        View the file list of a specified knowledge base, which includes lists of documents, spreadsheets, or images.

        docs en: https://www.coze.com/docs/developer_guides/list_knowledge_files
        docs zh: https://www.coze.cn/docs/developer_guides/list_knowledge_files


        :param dataset_id: The ID of the knowledge base.
        :param page_num: The page number for paginated queries. Default is 1, meaning the data return starts from the
        first page.
        :param page_size: The size of pagination. Default is 10, meaning that 10 data entries are returned per page.
        :return: list of Document
        """
        url = f"{self._base_url}/open_api/knowledge/document/list"
        headers = {"Agw-Js-Conv": "str"}

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "POST",
                url,
                headers=headers,
                json={
                    "dataset_id": dataset_id,
                    "page": i_page_num,
                    "size": i_page_size,
                },
                cast=_PrivateListDocumentsData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



import abc
import asyncio
import json
import queue
import sys
import threading
import traceback
from abc import ABC
from contextlib import asynccontextmanager, contextmanager
from enum import Enum
from typing import Any, Callable, Dict, List, Optional, Set

if sys.version_info >= (3, 8):
    # note: >=3.7,<3.8 not support asyncio
    from websockets import InvalidStatus
    from websockets.asyncio.client import ClientConnection as AsyncWebsocketClientConnection
    from websockets.asyncio.client import connect as asyncio_connect
else:
    # 警告: 当前Python版本不支持asyncio websockets
    # 如果Python版本小于3.8,则不支持异步websockets功能
    import warnings

    warnings.warn("asyncio websockets requires Python >= 3.8")

    class AsyncWebsocketClientConnection(object):
        def recv(self, *args, **kwargs):
            pass

        def send(self, *args, **kwargs):
            pass

        def close(self, *args, **kwargs):
            pass

    def asyncio_connect(*args, **kwargs):
        pass

    class InvalidStatus(object):
        pass


import websockets.sync.client
from pydantic import BaseModel

from cozepy import CozeAPIError
from cozepy.log import log_debug, log_error, log_info
from cozepy.model import CozeModel
from cozepy.request import Requester
from cozepy.util import get_methods, get_model_default, remove_url_trailing_slash
from cozepy.version import coze_client_user_agent, user_agent


class WebsocketsEventType(str, Enum):
    # common
    CLIENT_ERROR = "client_error"  # sdk error
    CLOSED = "closed"  # connection closed

    # error
    ERROR = "error"  # received error event

    # v1/audio/speech
    # req
    INPUT_TEXT_BUFFER_APPEND = "input_text_buffer.append"  # send text to server
    INPUT_TEXT_BUFFER_COMPLETE = (
        "input_text_buffer.complete"  # no text to send, after audio all received, can close connection
    )
    SPEECH_UPDATE = "speech.update"  # send speech config to server
    # resp
    # v1/audio/speech
    SPEECH_CREATED = "speech.created"  # after speech created
    INPUT_TEXT_BUFFER_COMPLETED = "input_text_buffer.completed"  # received `input_text_buffer.complete` event
    SPEECH_AUDIO_UPDATE = "speech.audio.update"  # received `speech.update` event
    SPEECH_AUDIO_COMPLETED = "speech.audio.completed"  # all audio received, can close connection

    # v1/audio/transcriptions
    # req
    INPUT_AUDIO_BUFFER_APPEND = "input_audio_buffer.append"  # send audio to server
    INPUT_AUDIO_BUFFER_COMPLETE = (
        "input_audio_buffer.complete"  # no audio to send, after text all received, can close connection
    )
    TRANSCRIPTIONS_UPDATE = "transcriptions.update"  # send transcriptions config to server
    # resp
    TRANSCRIPTIONS_CREATED = "transcriptions.created"  # after transcriptions created
    INPUT_AUDIO_BUFFER_COMPLETED = "input_audio_buffer.completed"  # received `input_audio_buffer.complete` event
    TRANSCRIPTIONS_MESSAGE_UPDATE = "transcriptions.message.update"  # received `transcriptions.update` event
    TRANSCRIPTIONS_MESSAGE_COMPLETED = "transcriptions.message.completed"  # all audio received, can close connection

    # v1/chat
    # req
    # INPUT_AUDIO_BUFFER_APPEND = "input_audio_buffer.append" # send audio to server
    # INPUT_AUDIO_BUFFER_COMPLETE = "input_audio_buffer.complete" # no audio send, start chat
    CHAT_UPDATE = "chat.update"  # send chat config to server
    CONVERSATION_CHAT_SUBMIT_TOOL_OUTPUTS = "conversation.chat.submit_tool_outputs"  # send tool outputs to server
    CONVERSATION_CHAT_CANCEL = "conversation.chat.cancel"  # send cancel chat to server
    CONVERSATION_MESSAGE_CREATE = "conversation.message.create"  # send text or string_object chat to server
    # resp
    CHAT_CREATED = "chat.created"
    CHAT_UPDATED = "chat.updated"
    # INPUT_AUDIO_BUFFER_COMPLETED = "input_audio_buffer.completed" # received `input_audio_buffer.complete` event
    CONVERSATION_CHAT_CREATED = "conversation.chat.created"  # audio ast completed, chat started
    CONVERSATION_CHAT_IN_PROGRESS = "conversation.chat.in_progress"
    CONVERSATION_MESSAGE_DELTA = "conversation.message.delta"  # get agent text message update
    CONVERSATION_CHAT_REQUIRES_ACTION = "conversation.chat.requires_action"  # need plugin submit
    CONVERSATION_AUDIO_TRANSCRIPT_COMPLETED = "conversation.audio_transcript.completed"
    CONVERSATION_MESSAGE_COMPLETED = "conversation.message.completed"
    CONVERSATION_AUDIO_DELTA = "conversation.audio.delta"  # get agent audio message update
    CONVERSATION_AUDIO_COMPLETED = "conversation.audio.completed"
    CONVERSATION_CHAT_COMPLETED = "conversation.chat.completed"  # all message received, can close connection
    CONVERSATION_CHAT_CANCELED = "conversation.chat.canceled"  # chat canceled


class WebsocketsEvent(CozeModel, ABC):
    class Detail(BaseModel):
        logid: Optional[str] = None

    event_type: WebsocketsEventType
    id: Optional[str] = None
    detail: Optional[Detail] = None


class WebsocketsErrorEvent(WebsocketsEvent):
    event_type: WebsocketsEventType = WebsocketsEventType.ERROR
    data: CozeAPIError


class InputAudio(BaseModel):
    format: Optional[str]
    codec: Optional[str]
    sample_rate: Optional[int]
    channel: Optional[int]
    bit_depth: Optional[int]


class OpusConfig(BaseModel):
    bitrate: Optional[int] = None
    use_cbr: Optional[bool] = None
    frame_size_ms: Optional[float] = None


class PCMConfig(BaseModel):
    sample_rate: Optional[int] = None


class OutputAudio(BaseModel):
    codec: Optional[str]
    pcm_config: Optional[PCMConfig] = None
    opus_config: Optional[OpusConfig] = None
    speech_rate: Optional[int] = None
    voice_id: Optional[str] = None


class WebsocketsBaseClient(abc.ABC):
    class State(str, Enum):
        """
        initialized, connecting, connected, closing, closed
        """

        INITIALIZED = "initialized"
        CONNECTING = "connecting"
        CONNECTED = "connected"
        CLOSING = "closing"
        CLOSED = "closed"

    def __init__(
        self,
        base_url: str,
        requester: Requester,
        path: str,
        query: Optional[Dict[str, str]] = None,
        on_event: Optional[Dict[WebsocketsEventType, Callable]] = None,
        wait_events: Optional[List[WebsocketsEventType]] = None,
        **kwargs,
    ):
        self._state = self.State.INITIALIZED
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._path = path
        self._ws_url = self._base_url + "/" + path
        if query:
            self._ws_url += "?" + "&".join([f"{k}={v}" for k, v in query.items()])
        self._on_event = on_event.copy() if on_event else {}
        self._headers = kwargs.get("headers")
        self._wait_events = wait_events.copy() if wait_events else []

        self._input_queue: queue.Queue[Optional[WebsocketsEvent]] = queue.Queue()
        self._ws: Optional[websockets.sync.client.ClientConnection] = None
        self._send_thread: Optional[threading.Thread] = None
        self._receive_thread: Optional[threading.Thread] = None
        self._completed_events: Set[WebsocketsEventType] = set()
        self._completed_event = threading.Event()

    @contextmanager
    def __call__(self):
        try:
            self.connect()
            yield self
        finally:
            self.close()

    def connect(self):
        if self._state != self.State.INITIALIZED:
            raise ValueError(f"Cannot connect in {self._state.value} state")
        self._state = self.State.CONNECTING
        headers = {
            "X-Coze-Client-User-Agent": coze_client_user_agent(),
            **(self._headers or {}),
        }

        self._requester.auth_header(headers)

        try:
            self._ws = websockets.sync.client.connect(
                self._ws_url,
                user_agent_header=user_agent(),
                additional_headers=headers,
            )
            self._state = self.State.CONNECTED
            log_info("[%s] connected to websocket", self._path)

            self._send_thread = threading.Thread(target=self._send_loop)
            self._receive_thread = threading.Thread(target=self._receive_loop)
            self._send_thread.start()
            self._receive_thread.start()
        except InvalidStatus as e:
            raise CozeAPIError(None, f"{e}", e.response.headers.get("x-tt-logid")) from e

    def wait(self, events: Optional[List[WebsocketsEventType]] = None, wait_all=True) -> None:
        if events is None:
            events = self._wait_events
        self._wait_completed(events, wait_all=wait_all)

    def on(self, event_type: WebsocketsEventType, handler: Callable):
        self._on_event[event_type] = handler

    def close(self) -> None:
        if self._state not in (self.State.CONNECTED, self.State.CONNECTING):
            return
        self._state = self.State.CLOSING
        self._close()
        self._state = self.State.CLOSED

    def _send_loop(self) -> None:
        try:
            while True:
                event = self._input_queue.get()
                self._send_event(event)
                self._input_queue.task_done()
        except Exception as e:
            self._handle_error(e)

    def _receive_loop(self) -> None:
        try:
            while True:
                if not self._ws:
                    log_debug("[%s] empty websocket conn, close", self._path)
                    break

                data = self._ws.recv()
                message = json.loads(data)
                event_type = message.get("event_type")
                log_debug("[%s] receive event, type=%s, event=%s", self._path, event_type, data)

                event = self._load_all_event(message)
                if event:
                    handler = self._on_event.get(event_type)
                    if handler:
                        handler(self, event)
                    self._completed_events.add(event_type)
                    self._completed_event.set()
        except Exception as e:
            self._handle_error(e)

    def _load_all_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        event_type = message.get("event_type") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.ERROR.value:
            code, msg = data.get("code") or 0, data.get("msg") or ""
            return WebsocketsErrorEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": CozeAPIError(code, msg, WebsocketsEvent.Detail.model_validate(detail).logid),
                }
            )
        return self._load_event(message)

    @abc.abstractmethod
    def _load_event(self, message: Dict) -> Optional[WebsocketsEvent]: ...

    def _wait_completed(self, events: List[WebsocketsEventType], wait_all: bool) -> None:
        while True:
            if wait_all:
                # 所有事件都需要完成
                if self._completed_events == set(events):
                    break
            else:
                # 任意一个事件完成即可
                if any(event in self._completed_events for event in events):
                    break
            self._completed_event.wait()
            self._completed_event.clear()

    def _handle_error(self, error: Exception) -> None:
        handler = self._on_event.get(WebsocketsEventType.ERROR)
        if handler:
            handler(self, error)
        else:
            raise error

    def _close(self) -> None:
        log_info("[%s] connect closed", self._path)
        if self._send_thread:
            self._send_thread.join()
        if self._receive_thread:
            self._receive_thread.join()

        if self._ws:
            self._ws.close()
            self._ws = None

        while not self._input_queue.empty():
            self._input_queue.get()

        handler = self._on_event.get(WebsocketsEventType.CLOSED)
        if handler:
            handler(self)

    def _send_event(self, event: Optional[WebsocketsEvent] = None) -> None:
        if not event or not self._ws:
            return
        log_debug("[%s] send event, type=%s", self._path, event.event_type.value)
        self._ws.send(event.model_dump_json())


def get_event_type_mapping(cls: Any) -> dict:
    res = {}
    method_list = get_methods(cls)
    for method in method_list:
        parameters = method.get("parameters", [])
        for param in parameters:
            if issubclass(param.annotation, WebsocketsEvent):
                event_type = get_model_default(param.annotation, "event_type")
                if event_type:
                    res[event_type] = method.get("function")
                    break
    return res


class WebsocketsBaseEventHandler(object):
    def on_client_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Client Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    def on_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    def on_closed(self, cli: "WebsocketsBaseClient"):
        pass

    def to_dict(self, origin: Optional[Dict[WebsocketsEventType, Callable]] = None):
        res = {
            WebsocketsEventType.CLIENT_ERROR: self.on_client_error,
            WebsocketsEventType.ERROR: self.on_error,
            WebsocketsEventType.CLOSED: self.on_closed,
        }

        res.update(get_event_type_mapping(self))
        res.update(origin or {})
        return res


class AsyncWebsocketsBaseClient(abc.ABC):
    class State(str, Enum):
        """
        initialized, connecting, connected, closing, closed
        """

        INITIALIZED = "initialized"
        CONNECTING = "connecting"
        CONNECTED = "connected"
        CLOSING = "closing"
        CLOSED = "closed"

    def __init__(
        self,
        base_url: str,
        requester: Requester,
        path: str,
        query: Optional[Dict[str, str]] = None,
        on_event: Optional[Dict[WebsocketsEventType, Callable]] = None,
        wait_events: Optional[List[WebsocketsEventType]] = None,
        **kwargs,
    ):
        self._state = self.State.INITIALIZED
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._path = path
        self._ws_url = self._base_url + "/" + path
        if query:
            self._ws_url += "?" + "&".join([f"{k}={v}" for k, v in query.items()])
        self._on_event = on_event.copy() if on_event else {}
        self._headers = kwargs.get("headers")
        self._wait_events = wait_events.copy() if wait_events else []

        self._input_queue: asyncio.Queue[Optional[WebsocketsEvent]] = asyncio.Queue()
        self._ws: Optional[AsyncWebsocketClientConnection] = None
        self._send_task: Optional[asyncio.Task] = None
        self._receive_task: Optional[asyncio.Task] = None

    @asynccontextmanager
    async def __call__(self):
        try:
            await self.connect()
            yield self
        finally:
            await self.close()

    async def connect(self):
        if self._state != self.State.INITIALIZED:
            raise ValueError(f"Cannot connect in {self._state.value} state")
        self._state = self.State.CONNECTING
        headers = {
            "X-Coze-Client-User-Agent": coze_client_user_agent(),
            **(self._headers or {}),
        }

        await self._requester.async_auth_header(headers)

        try:
            self._ws = await asyncio_connect(
                self._ws_url,
                user_agent_header=user_agent(),
                additional_headers=headers,
            )
            self._state = self.State.CONNECTED
            log_info("[%s] connected to websocket", self._path)

            self._send_task = asyncio.create_task(self._send_loop())
            self._receive_task = asyncio.create_task(self._receive_loop())
        except InvalidStatus as e:
            raise CozeAPIError(None, f"{e}", e.response.headers.get("x-tt-logid")) from e

    async def wait(self, events: Optional[List[WebsocketsEventType]] = None, wait_all=True) -> None:
        if events is None:
            events = self._wait_events
        await self._wait_completed(events, wait_all=wait_all)

    def on(self, event_type: WebsocketsEventType, handler: Callable):
        self._on_event[event_type] = handler

    async def close(self) -> None:
        if self._state not in (self.State.CONNECTED, self.State.CONNECTING):
            return
        self._state = self.State.CLOSING
        await self._close()
        self._state = self.State.CLOSED

    async def _send_loop(self) -> None:
        try:
            while True:
                event = await self._input_queue.get()
                await self._send_event(event)
                self._input_queue.task_done()
        except Exception as e:
            await self._handle_error(e)

    async def _receive_loop(self) -> None:
        try:
            while True:
                if not self._ws:
                    log_debug("[%s] empty websocket conn, close", self._path)
                    break

                data = await self._ws.recv()
                message = json.loads(data)
                event_type = message.get("event_type")
                log_debug("[%s] receive event, type=%s, event=%s", self._path, event_type, data)

                handler = self._on_event.get(event_type)
                event = self._load_all_event(message)
                if handler and event:
                    await handler(self, event)
        except Exception as e:
            await self._handle_error(e)

    def _load_all_event(self, message: Dict) -> Optional[WebsocketsEvent]:
        event_id = message.get("id") or ""
        event_type = message.get("event_type") or ""
        detail = WebsocketsEvent.Detail.model_validate(message.get("detail") or {})
        data = message.get("data") or {}
        if event_type == WebsocketsEventType.ERROR.value:
            code, msg = data.get("code") or 0, data.get("msg") or ""
            return WebsocketsErrorEvent.model_validate(
                {
                    "id": event_id,
                    "detail": detail,
                    "data": CozeAPIError(code, msg, WebsocketsEvent.Detail.model_validate(detail).logid),
                }
            )
        return self._load_event(message)

    @abc.abstractmethod
    def _load_event(self, message: Dict) -> Optional[WebsocketsEvent]: ...

    async def _wait_completed(self, wait_events: List[WebsocketsEventType], wait_all: bool) -> None:
        future: asyncio.Future[None] = asyncio.Future()
        completed_events = set()

        def _wrap_handler(event_type: WebsocketsEventType, original_handler):
            async def wrapped(client, event):
                # 先执行原始处理函数
                if original_handler:
                    await original_handler(client, event)

                # 再检查完成条件
                completed_events.add(event_type)
                if wait_all:
                    # 所有事件都需要完成
                    if completed_events == set(wait_events):
                        if not future.done():
                            future.set_result(None)
                else:
                    # 任意一个事件完成即可
                    if not future.done():
                        future.set_result(None)

            return wrapped

        # 为每个指定的事件类型包装处理函数
        origin_handlers = {}
        for event_type in wait_events:
            original_handler = self._on_event.get(event_type)
            origin_handlers[event_type] = original_handler
            self._on_event[event_type] = _wrap_handler(event_type, original_handler)

        try:
            # 等待直到满足完成条件
            await future
        finally:
            # 恢复所有原来的处理函数
            for event_type in wait_events:
                original_handler = origin_handlers.get(event_type)
                if original_handler:
                    self._on_event[event_type] = original_handler

    async def _handle_error(self, error: Exception) -> None:
        handler = self._on_event.get(WebsocketsEventType.ERROR)
        if handler:
            await handler(self, error)
        else:
            raise error

    async def _close(self) -> None:
        log_info("[%s] connect closed", self._path)
        if self._send_task:
            self._send_task.cancel()
        if self._receive_task:
            self._receive_task.cancel()

        if self._ws:
            await self._ws.close()
            self._ws = None

        while not self._input_queue.empty():
            await self._input_queue.get()

        handler = self._on_event.get(WebsocketsEventType.CLOSED)
        if handler:
            await handler(self)

    async def _send_event(self, event: Optional[WebsocketsEvent] = None) -> None:
        if not event or not self._ws:
            return
        if event.event_type == WebsocketsEventType.INPUT_AUDIO_BUFFER_APPEND:
            log_debug(
                "[%s] send event, type=%s, event=%s",
                self._path,
                event.event_type.value,
                json.dumps(event._dump_without_delta()),  # type: ignore
            )
        else:
            log_debug("[%s] send event, type=%s, event=%s", self._path, event.event_type.value, event.model_dump_json())
        await self._ws.send(event.model_dump_json())


class AsyncWebsocketsBaseEventHandler(object):
    async def on_client_error(self, cli: "WebsocketsBaseClient", e: Exception):
        log_error(f"Client Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    async def on_error(self, cli: "AsyncWebsocketsBaseClient", e: CozeAPIError):
        log_error(f"Error occurred: {str(e)}")
        log_error(f"Stack trace:\n{traceback.format_exc()}")

    async def on_closed(self, cli: "AsyncWebsocketsBaseClient"):
        pass

    def to_dict(self, origin: Optional[Dict[WebsocketsEventType, Callable]] = None):
        res = {
            WebsocketsEventType.CLIENT_ERROR: self.on_client_error,
            WebsocketsEventType.ERROR: self.on_error,
            WebsocketsEventType.CLOSED: self.on_closed,
        }

        res.update(get_event_type_mapping(self))
        res.update(origin or {})
        return res



from cozepy.request import Requester
from cozepy.util import http_base_url_to_ws, remove_url_trailing_slash

from .audio import AsyncWebsocketsAudioClient, WebsocketsAudioClient
from .chat import AsyncWebsocketsChatBuildClient, WebsocketsChatBuildClient


class WebsocketsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = http_base_url_to_ws(remove_url_trailing_slash(base_url))
        self._requester = requester

    @property
    def audio(self) -> WebsocketsAudioClient:
        return WebsocketsAudioClient(
            base_url=self._base_url,
            requester=self._requester,
        )

    @property
    def chat(self) -> WebsocketsChatBuildClient:
        return WebsocketsChatBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )


class AsyncWebsocketsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = http_base_url_to_ws(remove_url_trailing_slash(base_url))
        self._requester = requester

    @property
    def audio(self) -> AsyncWebsocketsAudioClient:
        return AsyncWebsocketsAudioClient(
            base_url=self._base_url,
            requester=self._requester,
        )

    @property
    def chat(self) -> AsyncWebsocketsChatBuildClient:
        return AsyncWebsocketsChatBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )



from typing import (
    TYPE_CHECKING,
    Any,
    List,
    Optional,
    Tuple,
    Type,
    TypeVar,
    Union,
    overload,
)

import httpx
from httpx import Response
from pydantic import BaseModel
from typing_extensions import Literal, get_args

from cozepy.config import DEFAULT_CONNECTION_LIMITS, DEFAULT_TIMEOUT
from cozepy.exception import COZE_PKCE_AUTH_ERROR_TYPE_ENUMS, CozeAPIError, CozePKCEAuthError, CozePKCEAuthErrorType
from cozepy.log import log_debug, log_warning
from cozepy.model import (
    AsyncIteratorHTTPResponse,
    FileHTTPResponse,
    HTTPRequest,
    IteratorHTTPResponse,
    ListResponse,
)
from cozepy.version import coze_client_user_agent, user_agent

if TYPE_CHECKING:
    from cozepy.auth import Auth

T = TypeVar("T", bound=BaseModel)


class SyncHTTPClient(httpx.Client):
    def __init__(self, **kwargs):
        kwargs.setdefault("timeout", DEFAULT_TIMEOUT)
        kwargs.setdefault("limits", DEFAULT_CONNECTION_LIMITS)
        kwargs.setdefault("follow_redirects", True)
        super().__init__(**kwargs)


class AsyncHTTPClient(httpx.AsyncClient):
    def __init__(self, **kwargs):
        kwargs.setdefault("timeout", DEFAULT_TIMEOUT)
        kwargs.setdefault("limits", DEFAULT_CONNECTION_LIMITS)
        kwargs.setdefault("follow_redirects", True)
        super().__init__(**kwargs)


class Requester(object):
    """
    http request helper class.
    """

    def __init__(
        self,
        auth: Optional["Auth"] = None,
        sync_client: Optional[SyncHTTPClient] = None,
        async_client: Optional[AsyncHTTPClient] = None,
    ):
        self._auth = auth
        self._sync_client = sync_client
        self._async_client = async_client

    def auth_header(self, headers: dict):
        if self._auth:
            self._auth.authentication(headers)

    async def async_auth_header(self, headers: dict):
        if self._auth:
            await self._auth.aauthentication(headers)

    def make_request(
        self,
        method: str,
        url: str,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        json: Optional[dict] = None,
        files: Optional[dict] = None,
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None] = None,
        data_field: str = "data",
        stream: bool = False,
    ) -> HTTPRequest:
        if headers is None:
            headers = {}
        headers["User-Agent"] = user_agent()
        headers["X-Coze-Client-User-Agent"] = coze_client_user_agent()
        if self._auth:
            self._auth.authentication(headers)

        log_debug(
            "request %s#%s sending, params=%s, json=%s, stream=%s, async=%s",
            method,
            url,
            params,
            json,
            stream,
            False,
        )

        return HTTPRequest(
            method=method,
            url=url,
            params=params,
            headers=headers,
            json_body=json,
            files=files,
            is_async=False,
            stream=stream,
            data_field=data_field,
            cast=cast,
        )

    async def amake_request(
        self,
        method: str,
        url: str,
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        json: Optional[dict] = None,
        files: Optional[dict] = None,
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None] = None,
        data_field: str = "data",
        stream: bool = False,
    ) -> HTTPRequest:
        if headers is None:
            headers = {}
        headers["User-Agent"] = user_agent()
        headers["X-Coze-Client-User-Agent"] = coze_client_user_agent()

        if self._auth:
            await self._auth.aauthentication(headers)

        log_debug(
            "request %s#%s sending, params=%s, json=%s, stream=%s, async=%s",
            method,
            url,
            params,
            json,
            stream,
            True,
        )

        return HTTPRequest(
            method=method,
            url=url,
            params=params,
            headers=headers,
            json_body=json,
            files=files,
            is_async=True,
            stream=stream,
            data_field=data_field,
            cast=cast,
        )

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[T],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> T: ...

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: List[Type[T]],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> List[T]: ...

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[ListResponse[T]],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> ListResponse[T]: ...

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[FileHTTPResponse],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> FileHTTPResponse: ...

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[True],
        cast: None,
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> IteratorHTTPResponse[str]: ...

    @overload
    def request(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: None,
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> None: ...

    def request(
        self,
        method: str,
        url: str,
        stream: Literal[True, False],
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None],
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        body: Optional[dict] = None,
        files: Optional[dict] = None,
        data_field: str = "data",
    ) -> Union[T, List[T], ListResponse[T], IteratorHTTPResponse[str], FileHTTPResponse, None]:
        """
        Send a request to the server.
        """
        method = method.upper()

        request = self.make_request(
            method,
            url,
            params=params,
            headers=headers,
            json=body,
            files=files,
            cast=cast,
            data_field=data_field,
            stream=stream,
        )

        return self.send(request)

    def send(
        self,
        request: HTTPRequest,
    ) -> Union[T, List[T], ListResponse[T], IteratorHTTPResponse[str], FileHTTPResponse, None]:
        """
        Send a request to the server.
        """
        return self._parse_response(
            method=request.method,
            url=request.url,
            is_async=False,
            response=self.sync_client.send(request.as_httpx, stream=request.stream),
            cast=request.cast,
            stream=request.stream,
            data_field=request.data_field,
        )

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[T],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> T: ...

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: List[Type[T]],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> List[T]: ...

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[ListResponse[T]],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> ListResponse[T]: ...

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: Type[FileHTTPResponse],
        params: dict = ...,
        headers: Optional[dict] = ...,
        body: dict = ...,
        files: dict = ...,
        data_field: str = ...,
    ) -> FileHTTPResponse: ...

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[False],
        cast: None,
        params: Optional[dict] = ...,
        headers: Optional[dict] = ...,
        body: Optional[dict] = ...,
        files: Optional[dict] = ...,
        data_field: str = ...,
    ) -> None: ...

    @overload
    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[True],
        cast: None,
        params: Optional[dict] = ...,
        headers: Optional[dict] = ...,
        body: Optional[dict] = ...,
        files: Optional[dict] = ...,
        data_field: str = ...,
    ) -> AsyncIteratorHTTPResponse[str]: ...

    async def arequest(
        self,
        method: str,
        url: str,
        stream: Literal[True, False],
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None],
        params: Optional[dict] = None,
        headers: Optional[dict] = None,
        body: Optional[dict] = None,
        files: Optional[dict] = None,
        data_field: str = "data",
    ) -> Union[T, List[T], ListResponse[T], AsyncIteratorHTTPResponse[str], FileHTTPResponse, None]:
        """
        Send a request to the server.
        """
        method = method.upper()
        request = await self.amake_request(
            method, url, params=params, headers=headers, json=body, files=files, stream=stream
        )

        response = await self.async_client.send(request.as_httpx, stream=stream)
        return self._parse_response(
            method, url, True, response=response, cast=cast, stream=stream, data_field=data_field
        )

    async def asend(
        self,
        request: HTTPRequest,
    ) -> Union[T, List[T], ListResponse[T], AsyncIteratorHTTPResponse[str], FileHTTPResponse, None]:
        return self._parse_response(
            method=request.method,
            url=request.url,
            is_async=True,
            response=await self.async_client.send(request.as_httpx, stream=request.stream),
            cast=request.cast,
            stream=request.stream,
            data_field=request.data_field,
        )

    @property
    def sync_client(self) -> "SyncHTTPClient":
        if self._sync_client is None:
            self._sync_client = SyncHTTPClient()
        return self._sync_client

    @property
    def async_client(self) -> "AsyncHTTPClient":
        if self._async_client is None:
            self._async_client = AsyncHTTPClient()
        return self._async_client

    @overload
    def _parse_response(
        self,
        method: str,
        url: str,
        is_async: Literal[False],
        response: httpx.Response,
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None],
        stream: bool = ...,
        data_field: str = ...,
    ) -> Union[T, List[T], ListResponse[T], IteratorHTTPResponse[str], FileHTTPResponse, None]: ...

    @overload
    def _parse_response(
        self,
        method: str,
        url: str,
        is_async: Literal[True],
        response: httpx.Response,
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None],
        stream: bool = ...,
        data_field: str = ...,
    ) -> Union[T, List[T], ListResponse[T], AsyncIteratorHTTPResponse[str], FileHTTPResponse, None]: ...

    def _parse_response(
        self,
        method: str,
        url: str,
        is_async: Literal[True, False],
        response: httpx.Response,
        cast: Union[Type[T], List[Type[T]], Type[ListResponse[T]], Type[FileHTTPResponse], None],
        stream: bool = False,
        data_field: str = "data",
    ) -> Union[
        T, List[T], ListResponse[T], IteratorHTTPResponse[str], AsyncIteratorHTTPResponse[str], FileHTTPResponse, None
    ]:
        # application/json
        # text/event-stream
        # audio/<xx>
        resp_content_type = response.headers.get("content-type")
        if resp_content_type:
            resp_content_type = resp_content_type.lower()
        logid = response.headers.get("x-tt-logid")
        if stream and "event-stream" in resp_content_type:
            if is_async:
                return AsyncIteratorHTTPResponse(response, response.aiter_lines())
            return IteratorHTTPResponse(response, response.iter_lines())

        if resp_content_type and "audio" in resp_content_type:
            return FileHTTPResponse(response)  # type: ignore

        code, msg, data = self._parse_requests_code_msg(method, url, response, data_field)

        if code is not None and code > 0:
            log_warning("request %s#%s failed, logid=%s, code=%s, msg=%s", method, url, logid, code, msg)
            raise CozeAPIError(code, msg, logid)
        elif code is None and msg != "":
            log_warning("request %s#%s failed, logid=%s, msg=%s", method, url, logid, msg)
            if msg in COZE_PKCE_AUTH_ERROR_TYPE_ENUMS:
                raise CozePKCEAuthError(CozePKCEAuthErrorType(msg), logid)
            raise CozeAPIError(code, msg, logid)
        if isinstance(cast, List):
            item_cast = cast[0]
            return [item_cast.model_validate(item) for item in data]
        elif hasattr(cast, "__origin__") and cast.__origin__ is ListResponse:  # type: ignore
            item_cast = get_args(cast)[0]
            return ListResponse(response, [item_cast.model_validate(item) for item in data])
        else:
            if cast is None:
                return None

            res = cast.model_validate(data) if data is not None else cast()  # type: ignore
            if hasattr(res, "_raw_response"):
                res._raw_response = response  # type: ignore
            return res  # type: ignore

    def _parse_requests_code_msg(
        self, method: str, url: str, response: Response, data_field: str = "data"
    ) -> Tuple[Optional[int], str, Any]:
        try:
            response.read()
            body = response.json()
            logid = response.headers.get("x-tt-logid")
            log_debug("request %s#%s responding, logid=%s, data=%s", method, url, logid, body)
        except Exception as e:  # noqa: E722
            raise CozeAPIError(
                response.status_code,
                response.text,
                response.headers.get("x-tt-logid"),
            ) from e

        if "code" in body and "msg" in body and int(body["code"]) > 0:
            return int(body["code"]), body["msg"], body.get(data_field)
        if "error_code" in body and body["error_code"] in COZE_PKCE_AUTH_ERROR_TYPE_ENUMS:
            return None, body["error_code"], None
        if "error_message" in body and body["error_message"] != "":
            return None, body["error_message"], None
        if data_field in body or "debug_url" in body:
            if "first_id" in body:
                return (
                    0,
                    "",
                    {
                        "first_id": body["first_id"],
                        "has_more": body["has_more"],
                        "last_id": body["last_id"],
                        "items": body["data"],
                    },
                )
            if "debug_url" in body:
                return (
                    0,
                    "",
                    {
                        "data": body.get(data_field),
                        "debug_url": body.get("debug_url") or "",
                        "execute_id": body.get("execute_id") or None,
                    },
                )
            return 0, "", body[data_field]
        if data_field == "data.data":
            return 0, "", body["data"]["data"]
        return 0, "", body



import json
import platform
import sys
from functools import lru_cache

VERSION = "0.14.0"


def get_os_version() -> str:
    os_name = platform.system().lower()
    if os_name == "darwin":
        os_name = "macos"

    if os_name == "macos":
        os_version = platform.mac_ver()[0]
    elif os_name == "windows":
        os_version = platform.win32_ver()[0]
    elif os_name == "linux":
        try:
            import distro

            os_version = distro.version(pretty=False, best=True)
        except ImportError:
            os_version = platform.release()
    else:
        os_version = platform.release()

    return os_version


@lru_cache(maxsize=1)
def user_agent():
    python_version = ".".join(map(str, sys.version_info[:2]))

    os_name = platform.system().lower()
    os_version = get_os_version()

    return f"cozepy/{VERSION} python/{python_version} {os_name}/{os_version}".lower()


@lru_cache(maxsize=1)
def coze_client_user_agent() -> str:
    ua = {
        "version": VERSION,
        "lang": "python",
        "lang_version": ".".join(map(str, sys.version_info[:2])),
        "os_name": platform.system().lower(),
        "os_version": get_os_version(),
    }

    return json.dumps(ua)



from enum import IntEnum
from typing import List, Optional

from cozepy.datasets.documents import DocumentSourceType
from cozepy.model import AsyncNumberPaged, CozeModel, HTTPRequest, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class PhotoStatus(IntEnum):
    IN_PROCESSING = 0  # In processing
    COMPLETED = 1  # Completed
    PROCESSING_FAILED = 9  # Processing failed


class Photo(CozeModel):
    document_id: str  # The ID of the image.
    url: str = ""  # The URL of the image.
    name: str = ""  # The name of the image.
    size: int = 0  # The size of the image, in bytes.
    type: str = ""  # File format, i.e., file extension, such as jpg, png.
    status: PhotoStatus  # Status of the file. Values include: 0: In processing, 1: Completed, 9: Processing failed, it is recommended to re-upload
    caption: str = ""  # Image description information.
    creator_id: str = ""  # The ID of the creator.
    source_type: DocumentSourceType  # The source type of the image.
    create_time: int = 0  # The creation time of the image.
    update_time: int = 0  # The update time of the image.


class UpdateImageRes(CozeModel):
    pass


class _PrivateListPhotosData(CozeModel, NumberPagedResponse[Photo]):
    photo_infos: List[Photo]
    total_count: int

    def get_total(self) -> Optional[int]:
        return self.total_count

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[Photo]:
        return self.photo_infos


class DatasetsImagesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def update(
        self,
        *,
        dataset_id: str,
        document_id: str,
        caption: str,
        **kwargs,
    ) -> UpdateImageRes:
        """
        Call this API to update the description of images in the knowledge base.

        docs en: https://www.coze.com/docs/developer_guides/update_image_caption
        docs zh: https://www.coze.cn/docs/developer_guides/update_image_caption

        :param dataset_id: The ID of the knowledge base.
        :param document_id: The ID of the image.
        :param caption: The description of the image.
        :return: None
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/images/{document_id}"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "caption": caption,
        }
        return self._requester.request("put", url, False, cast=UpdateImageRes, headers=headers, body=body)

    def list(
        self,
        *,
        dataset_id: str,
        keyword: Optional[str] = None,
        has_caption: Optional[bool] = None,
        page_num: int = 1,
        page_size: int = 10,
        **kwargs,
    ) -> NumberPaged[Photo]:
        """
        Call this API to view detailed information of images in the image knowledge base.
        When viewing images, it is supported to filter through image annotations.

        docs en: https://www.coze.com/docs/developer_guides/get_images
        docs zh: https://www.coze.cn/docs/developer_guides/get_images

        :param dataset_id: The ID of the knowledge base.
        :param keyword: The keyword to filter images.
        :param has_caption: Whether the image has a caption.
        :param page_num: The page number for paginated queries. Default is 1, meaning the data return starts from the
        first page.
        :param page_size: The size of pagination. Default is 10, meaning that 10 data entries are returned per page. The value range is 1~299, with a default of 10.
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/images"
        headers: Optional[dict] = kwargs.get("headers")

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "get",
                url,
                headers=headers,
                params={
                    "page_num": i_page_num,
                    "page_size": i_page_size,
                    "keyword": keyword,
                    "has_caption": has_caption,
                },
                cast=_PrivateListPhotosData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncDatasetsImagesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def update(
        self,
        *,
        dataset_id: str,
        document_id: str,
        caption: str,
        **kwargs,
    ) -> UpdateImageRes:
        """
        Call this API to update the description of images in the knowledge base.

        docs en: https://www.coze.com/docs/developer_guides/update_image_caption
        docs zh: https://www.coze.cn/docs/developer_guides/update_image_caption

        :param dataset_id: The ID of the knowledge base.
        :param document_id: The ID of the image.
        :param caption: The description of the image.
        :return: None
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/images/{document_id}"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "caption": caption,
        }
        return await self._requester.arequest("put", url, False, cast=UpdateImageRes, headers=headers, body=body)

    async def list(
        self,
        *,
        dataset_id: str,
        keyword: Optional[str] = None,
        has_caption: Optional[bool] = None,
        page_num: int = 1,
        page_size: int = 10,
        **kwargs,
    ) -> AsyncNumberPaged[Photo]:
        """
        Call this API to view detailed information of images in the image knowledge base.
        When viewing images, it is supported to filter through image annotations.

        docs en: https://www.coze.com/docs/developer_guides/get_images
        docs zh: https://www.coze.cn/docs/developer_guides/get_images

        :param dataset_id: The ID of the knowledge base.
        :param keyword: The keyword to filter images.
        :param has_caption: Whether the image has a caption.
        :param page_num: The page number for paginated queries. Default is 1, meaning the data return starts from the
        first page.
        :param page_size: The size of pagination. Default is 10, meaning that 10 data entries are returned per page. The value range is 1~299, with a default of 10.
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/images"
        headers: Optional[dict] = kwargs.get("headers")

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "get",
                url,
                headers=headers,
                params={
                    "page_num": i_page_num,
                    "page_size": i_page_size,
                    "keyword": keyword,
                    "has_caption": has_caption,
                },
                cast=_PrivateListPhotosData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



from cozepy.request import Requester

from .speech import AsyncWebsocketsAudioSpeechBuildClient, WebsocketsAudioSpeechBuildClient
from .transcriptions import AsyncWebsocketsAudioTranscriptionsBuildClient, WebsocketsAudioTranscriptionsBuildClient


class WebsocketsAudioClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = base_url
        self._requester = requester

    @property
    def transcriptions(self) -> "WebsocketsAudioTranscriptionsBuildClient":
        return WebsocketsAudioTranscriptionsBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )

    @property
    def speech(self) -> "WebsocketsAudioSpeechBuildClient":
        return WebsocketsAudioSpeechBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )


class AsyncWebsocketsAudioClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = base_url
        self._requester = requester

    @property
    def transcriptions(self) -> "AsyncWebsocketsAudioTranscriptionsBuildClient":
        return AsyncWebsocketsAudioTranscriptionsBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )

    @property
    def speech(self) -> "AsyncWebsocketsAudioSpeechBuildClient":
        return AsyncWebsocketsAudioSpeechBuildClient(
            base_url=self._base_url,
            requester=self._requester,
        )



from typing import Any, AsyncIterator, Dict, List, Optional

from cozepy.chat import (
    ChatEvent,
    Message,
    _async_chat_stream_handler,
    _sync_chat_stream_handler,
)
from cozepy.model import AsyncIteratorHTTPResponse, AsyncStream, IteratorHTTPResponse, Stream
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash


class WorkflowsChatClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def stream(
        self,
        *,
        workflow_id: str,
        additional_messages: Optional[List[Message]] = None,
        parameters: Optional[Dict[str, Any]] = None,
        app_id: Optional[str] = None,
        bot_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        ext: Optional[Dict[str, str]] = None,
        **kwargs,
    ) -> Stream[ChatEvent]:
        """
        Call the Chat API with streaming to send messages to a published Coze bot.

        docs en: https://www.coze.com/docs/developer_guides/workflow_chat
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_chat

        :param workflow_id: The ID of the workflow that the API interacts with.
        :param additional_messages: Additional information for the conversation. You can pass the user's query for this
        conversation through this field. The array length is limited to 50, meaning up to 50 messages can be input.
        :param parameters: The parameters for the workflow.
        :param app_id: The ID of the app that the API interacts with.
        :param bot_id: The ID of the bot that the API interacts with.
        :param conversation_id: Indicate which conversation the chat is taking place in.
        :param ext: The extended information for the workflow.
        :return: iterator of ChatEvent
        """
        return self._create(
            workflow_id=workflow_id,
            additional_messages=additional_messages,
            parameters=parameters,
            app_id=app_id,
            bot_id=bot_id,
            conversation_id=conversation_id,
            ext=ext,
            **kwargs,
        )

    def _create(
        self,
        *,
        workflow_id: str,
        additional_messages: Optional[List[Message]] = None,
        parameters: Optional[Dict[str, Any]] = None,
        app_id: Optional[str] = None,
        bot_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        ext: Optional[Dict[str, str]] = None,
        **kwargs,
    ) -> Stream[ChatEvent]:
        url = f"{self._base_url}/v1/workflows/chat"
        body = remove_none_values(
            {
                "workflow_id": workflow_id,
                "additional_messages": [i.model_dump() for i in additional_messages] if additional_messages else [],
                "parameters": parameters,
                "app_id": app_id,
                "bot_id": bot_id,
                "conversation_id": conversation_id,
                "ext": ext,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")
        response: IteratorHTTPResponse[str] = self._requester.request(
            "post",
            url,
            True,
            None,
            headers=headers,
            body=body,
        )
        return Stream(
            response._raw_response,
            response.data,
            fields=["event", "data"],
            handler=_sync_chat_stream_handler,
        )


class AsyncWorkflowsChatClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def stream(
        self,
        *,
        workflow_id: str,
        additional_messages: Optional[List[Message]] = None,
        parameters: Optional[Dict[str, Any]] = None,
        app_id: Optional[str] = None,
        bot_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        ext: Optional[Dict[str, str]] = None,
        **kwargs,
    ) -> AsyncIterator[ChatEvent]:
        """
        Call the Chat API with streaming to send messages to a published Coze bot.

        docs en: https://www.coze.com/docs/developer_guides/workflow_chat
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_chat

        :param workflow_id: The ID of the workflow that the API interacts with.
        :param additional_messages: Additional information for the conversation. You can pass the user's query for this
        conversation through this field. The array length is limited to 50, meaning up to 50 messages can be input.
        :param parameters: The parameters for the workflow.
        :param app_id: The ID of the app that the API interacts with.
        :param bot_id: The ID of the bot that the API interacts with.
        :param conversation_id: Indicate which conversation the chat is taking place in.
        :param ext: The extended information for the workflow.
        :return: iterator of ChatEvent
        """
        async for item in await self._create(
            workflow_id=workflow_id,
            additional_messages=additional_messages,
            parameters=parameters,
            app_id=app_id,
            bot_id=bot_id,
            conversation_id=conversation_id,
            ext=ext,
            **kwargs,
        ):
            yield item

    async def _create(
        self,
        *,
        workflow_id: str,
        additional_messages: Optional[List[Message]] = None,
        parameters: Optional[Dict[str, Any]] = None,
        app_id: Optional[str] = None,
        bot_id: Optional[str] = None,
        conversation_id: Optional[str] = None,
        ext: Optional[Dict[str, str]] = None,
        **kwargs,
    ) -> AsyncIterator[ChatEvent]:
        url = f"{self._base_url}/v1/workflows/chat"
        body = remove_none_values(
            {
                "workflow_id": workflow_id,
                "additional_messages": [i.model_dump() for i in additional_messages] if additional_messages else [],
                "parameters": parameters,
                "app_id": app_id,
                "bot_id": bot_id,
                "conversation_id": conversation_id,
                "ext": ext,
            }
        )
        headers: Optional[dict] = kwargs.get("headers")
        resp: AsyncIteratorHTTPResponse[str] = await self._requester.arequest(
            "post",
            url,
            True,
            None,
            headers=headers,
            body=body,
        )

        return AsyncStream(
            resp.data, fields=["event", "data"], handler=_async_chat_stream_handler, raw_response=resp._raw_response
        )



from typing import Optional

from cozepy.model import CozeModel
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class User(CozeModel):
    user_id: str
    user_name: str
    nick_name: str
    avatar_url: str


class UsersClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def me(self, **kwargs) -> User:
        url = f"{self._base_url}/v1/users/me"
        headers: Optional[dict] = kwargs.get("headers")
        return self._requester.request("get", url, False, User, headers=headers)


class AsyncUsersClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def me(self, **kwargs) -> User:
        url = f"{self._base_url}/v1/users/me"
        headers: Optional[dict] = kwargs.get("headers")
        return await self._requester.arequest("get", url, False, User, headers=headers)



from enum import Enum
from typing import TYPE_CHECKING, Any, AsyncIterator, Dict, Optional

import httpx

from cozepy.model import AsyncIteratorHTTPResponse, AsyncStream, CozeModel, IteratorHTTPResponse, Stream
from cozepy.request import Requester
from cozepy.util import remove_none_values, remove_url_trailing_slash

if TYPE_CHECKING:
    from .run_histories import AsyncWorkflowsRunsRunHistoriesClient, WorkflowsRunsRunHistoriesClient


class WorkflowRunResult(CozeModel):
    debug_url: str

    # Workflow execution result, usually a JSON serialized string. In some scenarios, a
    # string with a non-JSON structure may be returned.
    data: Optional[str] = ""

    # Execution ID of asynchronous execution. Only returned when the workflow is executed
    # asynchronously (is_async=true). You can use execute_id to call the Query Workflow
    # Asynchronous Execution Result API to obtain the final execution result of the workflow.
    execute_id: Optional[str] = ""


class WorkflowEventType(str, Enum):
    # The output message from the workflow node, such as the output message from
    # the message node or end node. You can view the specific message content in data.
    # 工作流节点输出消息，例如消息节点、结束节点的输出消息。可以在 data 中查看具体的消息内容。
    MESSAGE = "Message"

    # An error has occurred. You can view the error_code and error_message in data to
    # troubleshoot the issue.
    # 报错。可以在 data 中查看 error_code 和 error_message，排查问题。
    ERROR = "Error"

    # End. Indicates the end of the workflow execution, where data is empty.
    # 结束。表示工作流执行结束，此时 data 为空。
    DONE = "Done"

    # Interruption. Indicates the workflow has been interrupted, where the data field
    # contains specific interruption information.
    # 中断。表示工作流中断，此时 data 字段中包含具体的中断信息。
    INTERRUPT = "Interrupt"


class WorkflowEventMessage(CozeModel):
    # The content of the streamed output message.
    # 流式输出的消息内容。
    content: str

    # The name of the node that outputs the message, such as the message node or end node.
    # 输出消息的节点名称，例如消息节点、结束节点。
    node_title: str

    # The message ID of this message within the node, starting at 0, for example, the 5th message of the message node.
    # 此消息在节点中的消息 ID，从 0 开始计数，例如消息节点的第 5 条消息。
    node_seq_id: str

    # Whether the current message is the last data packet for this node.
    # 当前消息是否为此节点的最后一个数据包。
    node_is_finish: bool

    # Additional fields.
    # 额外字段。
    ext: Optional[Dict[str, Any]] = None


class WorkflowEventInterruptData(CozeModel):
    # The workflow interruption event ID, which should be passed back when resuming the workflow.
    # 工作流中断事件 ID，恢复运行时应回传此字段。
    event_id: str

    # The type of workflow interruption, which should be passed back when resuming the workflow.
    # 工作流中断类型，恢复运行时应回传此字段。
    type: int


class WorkflowEventInterrupt(CozeModel):
    # The content of interruption event.
    # 中断控制内容。
    interrupt_data: WorkflowEventInterruptData

    # The name of the node that outputs the message, such as "Question".
    # 输出消息的节点名称，例如“问答”。
    node_title: str


class WorkflowEventError(CozeModel):
    # Status code. 0 represents a successful API call. Other values indicate that the call has failed. You can
    # determine the detailed reason for the error through the error_message field.
    # 调用状态码。0 表示调用成功。其他值表示调用失败。你可以通过 error_message 字段判断详细的错误原因。
    error_code: int

    # Status message. You can get detailed error information when the API call fails.
    # 状态信息。API 调用失败时可通过此字段查看详细错误信息。
    error_message: str


class WorkflowEvent(CozeModel):
    # The event ID of this message in the interface response. It starts from 0.
    id: int

    # The current streaming data packet event.
    event: WorkflowEventType

    message: Optional[WorkflowEventMessage] = None

    interrupt: Optional[WorkflowEventInterrupt] = None

    error: Optional[WorkflowEventError] = None


def _workflow_stream_handler(
    data: Dict[str, str], raw_response: httpx.Response, is_async: bool = False
) -> WorkflowEvent:
    id = int(data["id"])
    event = data["event"]
    event_data = data["data"]  # type: str
    if event == WorkflowEventType.DONE:
        if is_async:
            raise StopAsyncIteration
        raise StopIteration
    elif event == WorkflowEventType.MESSAGE:
        return WorkflowEvent(
            id=id,
            event=event,
            message=WorkflowEventMessage.model_validate_json(event_data),
        )
    elif event == WorkflowEventType.ERROR:
        return WorkflowEvent(id=id, event=event, error=WorkflowEventError.model_validate_json(event_data))
    elif event == WorkflowEventType.INTERRUPT:
        return WorkflowEvent(
            id=id,
            event=event,
            interrupt=WorkflowEventInterrupt.model_validate_json(event_data),
        )
    else:
        raise ValueError(f"invalid workflows.event: {event}, {event_data}")


def _sync_workflow_stream_handler(data: Dict[str, str], raw_response: httpx.Response) -> WorkflowEvent:
    return _workflow_stream_handler(data, raw_response=raw_response, is_async=False)


def _async_workflow_stream_handler(data: Dict[str, str], raw_response: httpx.Response) -> WorkflowEvent:
    return _workflow_stream_handler(data, raw_response=raw_response, is_async=True)


class WorkflowsRunsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._run_histories: Optional[WorkflowsRunsRunHistoriesClient] = None

    def create(
        self,
        *,
        workflow_id: str,
        parameters: Optional[Dict[str, Any]] = None,
        bot_id: Optional[str] = None,
        app_id: Optional[str] = None,
        is_async: bool = False,
        ext: Optional[Dict[str, Any]] = None,
    ) -> WorkflowRunResult:
        """
        Run the published workflow.
        This API is in non-streaming response mode. For nodes that support streaming output,
        you should run the API Run workflow (streaming response) to obtain streaming responses.

        docs en: https://www.coze.com/docs/developer_guides/workflow_run
        docs cn: https://www.coze.cn/docs/developer_guides/workflow_run

        :param workflow_id: The ID of the workflow, which should have been published.
        :param parameters: Input parameters and their values for the starting node of the workflow. You can view the
        list of parameters on the arrangement page of the specified workflow.
        :param bot_id: The associated Bot ID required for some workflow executions,
        such as workflows with database nodes, variable nodes, etc.
        :param app_id: The app_id is required for some workflow executions,
        :param is_async: Whether to run asynchronously.
        :param ext: Used to specify some additional fields in the format of Map[String][String].
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/run"
        body = {
            "workflow_id": workflow_id,
            "parameters": parameters,
            "bot_id": bot_id,
            "app_id": app_id,
            "is_async": is_async,
            "ext": ext,
        }
        return self._requester.request("post", url, False, WorkflowRunResult, body=remove_none_values(body))

    def stream(
        self,
        *,
        workflow_id: str,
        parameters: Optional[Dict[str, Any]] = None,
        bot_id: Optional[str] = None,
        app_id: Optional[str] = None,
        ext: Optional[Dict[str, Any]] = None,
    ) -> Stream[WorkflowEvent]:
        """
        Execute the published workflow with a streaming response method.

        docs en: https://www.coze.com/docs/developer_guides/workflow_stream_run
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_stream_run

        :param workflow_id: The ID of the workflow, which should have been published.
        :param parameters: Input parameters and their values for the starting node of the workflow. You can view the
        list of parameters on the arrangement page of the specified workflow.
        :param bot_id: The associated Bot ID required for some workflow executions,
        such as workflows with database nodes, variable nodes, etc.
        :param app_id: The app_id is required for some workflow executions,
        :param ext: Used to specify some additional fields in the format of Map[String][String].
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/stream_run"
        body = {
            "workflow_id": workflow_id,
            "parameters": parameters,
            "bot_id": bot_id,
            "app_id": app_id,
            "ext": ext,
        }
        resp: IteratorHTTPResponse[str] = self._requester.request(
            "post",
            url,
            True,
            None,
            body=remove_none_values(body),
        )
        return Stream(
            resp._raw_response, resp.data, fields=["id", "event", "data"], handler=_sync_workflow_stream_handler
        )

    def resume(
        self,
        *,
        workflow_id: str,
        event_id: str,
        resume_data: str,
        interrupt_type: int,
    ) -> Stream[WorkflowEvent]:
        """
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_resume

        :param workflow_id: The ID of the workflow, which should have been published.
        :param event_id:
        :param resume_data:
        :param interrupt_type:
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/stream_resume"
        body = {
            "workflow_id": workflow_id,
            "event_id": event_id,
            "resume_data": resume_data,
            "interrupt_type": interrupt_type,
        }
        resp: IteratorHTTPResponse[str] = self._requester.request(
            "post",
            url,
            True,
            None,
            body=body,
        )
        return Stream(
            resp._raw_response, resp.data, fields=["id", "event", "data"], handler=_sync_workflow_stream_handler
        )

    @property
    def run_histories(self) -> "WorkflowsRunsRunHistoriesClient":
        if not self._run_histories:
            from .run_histories import WorkflowsRunsRunHistoriesClient

            self._run_histories = WorkflowsRunsRunHistoriesClient(self._base_url, self._requester)
        return self._run_histories


class AsyncWorkflowsRunsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

        self._run_histories: Optional[AsyncWorkflowsRunsRunHistoriesClient] = None

    async def create(
        self,
        *,
        workflow_id: str,
        parameters: Optional[Dict[str, Any]] = None,
        bot_id: Optional[str] = None,
        app_id: Optional[str] = None,
        is_async: bool = False,
        ext: Optional[Dict[str, Any]] = None,
    ) -> WorkflowRunResult:
        """
        Run the published workflow.
        This API is in non-streaming response mode. For nodes that support streaming output,
        you should run the API Run workflow (streaming response) to obtain streaming responses.

        docs en: https://www.coze.com/docs/developer_guides/workflow_run
        docs cn: https://www.coze.cn/docs/developer_guides/workflow_run

        :param workflow_id: The ID of the workflow, which should have been published.
        :param parameters: Input parameters and their values for the starting node of the workflow. You can view the
        list of parameters on the arrangement page of the specified workflow.
        :param bot_id: The associated Bot ID required for some workflow executions,
        such as workflows with database nodes, variable nodes, etc.
        :param app_id: The app_id is required for some workflow executions,
        :param is_async: Whether to run asynchronously.
        :param ext: Used to specify some additional fields in the format of Map[String][String].
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/run"
        body = {
            "workflow_id": workflow_id,
            "parameters": parameters,
            "bot_id": bot_id,
            "app_id": app_id,
            "is_async": is_async,
            "ext": ext,
        }
        return await self._requester.arequest("post", url, False, WorkflowRunResult, body=remove_none_values(body))

    async def stream(
        self,
        *,
        workflow_id: str,
        parameters: Optional[Dict[str, Any]] = None,
        bot_id: Optional[str] = None,
        app_id: Optional[str] = None,
        ext: Optional[Dict[str, Any]] = None,
    ) -> AsyncIterator[WorkflowEvent]:
        """
        Execute the published workflow with a streaming response method.

        docs en: https://www.coze.com/docs/developer_guides/workflow_stream_run
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_stream_run

        :param workflow_id: The ID of the workflow, which should have been published.
        :param parameters: Input parameters and their values for the starting node of the workflow. You can view the
        list of parameters on the arrangement page of the specified workflow.
        :param bot_id: The associated Bot ID required for some workflow executions,
        such as workflows with database nodes, variable nodes, etc.
        :param app_id: The app_id is required for some workflow executions,
        :param ext: Used to specify some additional fields in the format of Map[String][String].
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/stream_run"
        body = {
            "workflow_id": workflow_id,
            "parameters": parameters,
            "bot_id": bot_id,
            "app_id": app_id,
            "ext": ext,
        }
        resp: AsyncIteratorHTTPResponse[str] = await self._requester.arequest(
            "post",
            url,
            True,
            None,
            body=remove_none_values(body),
        )
        async for item in AsyncStream(
            resp.data,
            fields=["id", "event", "data"],
            handler=_async_workflow_stream_handler,
            raw_response=resp._raw_response,
        ):
            yield item

    async def resume(
        self,
        *,
        workflow_id: str,
        event_id: str,
        resume_data: str,
        interrupt_type: int,
    ) -> AsyncIterator[WorkflowEvent]:
        """
        docs zh: https://www.coze.cn/docs/developer_guides/workflow_resume

        :param workflow_id: The ID of the workflow, which should have been published.
        :param event_id:
        :param resume_data:
        :param interrupt_type:
        :return: The result of the workflow execution
        """
        url = f"{self._base_url}/v1/workflow/stream_resume"
        body = {
            "workflow_id": workflow_id,
            "event_id": event_id,
            "resume_data": resume_data,
            "interrupt_type": interrupt_type,
        }
        resp: AsyncIteratorHTTPResponse[str] = await self._requester.arequest(
            "post",
            url,
            True,
            None,
            body=body,
        )
        async for item in AsyncStream(
            resp.data,
            fields=["id", "event", "data"],
            handler=_async_workflow_stream_handler,
            raw_response=resp._raw_response,
        ):
            yield item

    @property
    def run_histories(self) -> "AsyncWorkflowsRunsRunHistoriesClient":
        if not self._run_histories:
            from .run_histories import AsyncWorkflowsRunsRunHistoriesClient

            self._run_histories = AsyncWorkflowsRunsRunHistoriesClient(self._base_url, self._requester)
        return self._run_histories



from enum import Enum
from typing import List, Optional

from cozepy.model import AsyncNumberPaged, CozeModel, HTTPRequest, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class WorkspaceRoleType(str, Enum):
    OWNER = "owner"
    ADMIN = "admin"
    MEMBER = "member"


class WorkspaceType(str, Enum):
    PERSONAL = "personal"
    TEAM = "team"


class Workspace(CozeModel):
    # workspace id
    id: str
    # workspace name
    name: str
    # workspace icon url
    icon_url: str
    # user in workspace role type
    role_type: WorkspaceRoleType
    # workspace type
    workspace_type: WorkspaceType


class _PrivateListWorkspacesData(CozeModel, NumberPagedResponse[Workspace]):
    total_count: int
    workspaces: List[Workspace]

    def get_total(self) -> Optional[int]:
        return self.total_count

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[Workspace]:
        return self.workspaces


class WorkspacesClient(object):
    """
    Bot class.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def list(self, *, page_num: int = 1, page_size: int = 20, headers=None) -> NumberPaged[Workspace]:
        url = f"{self._base_url}/v1/workspaces"

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                headers=headers,
                params={
                    "page_size": i_page_size,
                    "page_num": i_page_num,
                },
                cast=_PrivateListWorkspacesData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )


class AsyncWorkspacesClient(object):
    """
    Bot class.
    """

    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def list(self, *, page_num: int = 1, page_size: int = 20, headers=None) -> AsyncNumberPaged[Workspace]:
        url = f"{self._base_url}/v1/workspaces"

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                headers=headers,
                params={
                    "page_size": i_page_size,
                    "page_num": i_page_num,
                },
                cast=_PrivateListWorkspacesData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )



from enum import Enum
from typing import Optional

from cozepy.model import CozeModel
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash


class TemplateEntityType(str, Enum):
    AGENT = "agent"


class TemplateDuplicateResp(CozeModel):
    entity_id: str
    entity_type: TemplateEntityType


class TemplatesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    def duplicate(
        self, *, template_id: str, workspace_id: str, name: Optional[str] = None, **kwargs
    ) -> TemplateDuplicateResp:
        url = f"{self._base_url}/v1/templates/{template_id}/duplicate"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "workspace_id": workspace_id,
            "name": name,
        }
        return self._requester.request("post", url, False, TemplateDuplicateResp, headers=headers, body=body)


class AsyncTemplatesClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester

    async def duplicate(
        self, *, template_id: str, workspace_id: str, name: Optional[str] = None, **kwargs
    ) -> TemplateDuplicateResp:
        url = f"{self._base_url}/v1/templates/{template_id}/duplicate"
        headers: Optional[dict] = kwargs.get("headers")
        body = {
            "workspace_id": workspace_id,
            "name": name,
        }
        return await self._requester.arequest("post", url, False, TemplateDuplicateResp, headers=headers, body=body)



from enum import IntEnum
from typing import TYPE_CHECKING, List, Optional

from cozepy.datasets.documents import DocumentChunkStrategy, DocumentFormatType, DocumentStatus, DocumentUpdateType
from cozepy.model import AsyncNumberPaged, CozeModel, HTTPRequest, ListResponse, NumberPaged, NumberPagedResponse
from cozepy.request import Requester
from cozepy.util import remove_url_trailing_slash

if TYPE_CHECKING:
    from .documents import AsyncDatasetsDocumentsClient, DatasetsDocumentsClient
    from .images import AsyncDatasetsImagesClient, DatasetsImagesClient


class CreateDatasetResp(CozeModel):
    dataset_id: str


class DatasetStatus(IntEnum):
    ENABLED = 1
    DISABLED = 3


class Dataset(CozeModel):
    dataset_id: str  # The ID of the dataset
    name: str  # The name of the dataset
    description: str  # The description of the dataset
    space_id: str  # The ID of the space that the dataset belongs to
    status: DatasetStatus  # The status of the dataset, 1: enable, 3: disable
    format_type: DocumentFormatType  # The format type of the dataset, 0: text, 1: table, 2: image
    can_edit: bool = False  # Whether the dataset can be edited
    icon_url: str = ""  # The icon URL of the dataset
    doc_count: int = 0  # The count of the documents in the dataset
    file_list: List[str] = []  # The list of files in the dataset
    hit_count: int = 0  # The count of the hits in the dataset
    bot_used_count: int = 0  # The count of the bots used in the dataset
    slice_count: int = 0  # The count of the segments in the dataset
    all_file_size: int = 0  # The total size of all files in the dataset
    chunk_strategy: Optional[DocumentChunkStrategy] = None  # The chunk strategy of the dataset
    failed_file_list: List[str] = []  # The list of files that failed to be uploaded in the dataset
    processing_file_list: List[str] = []  # The list of files that are being processed in the dataset
    processing_file_id_list: List[str] = []  # The list of file IDs that are being processed in the dataset
    avatar_url: str = ""  # The avatar URL of the dataset creator
    creator_id: str = ""  # The ID of the dataset creator
    creator_name: str = ""  # The name of the dataset creator
    create_time: int = 0  # The creation second-level timestamp of the dataset
    update_time: int = 0  # The update second-level timestamp of the dataset


class _PrivateListDatasetsData(CozeModel, NumberPagedResponse[Dataset]):
    total_count: int
    dataset_list: List[Dataset]

    def get_total(self) -> Optional[int]:
        return self.total_count

    def get_has_more(self) -> Optional[bool]:
        return None

    def get_items(self) -> List[Dataset]:
        return self.dataset_list


class DocumentProgress(CozeModel):
    document_id: str = ""  # File ID.
    url: str = ""  # File address.
    size: int = 0  # The size of the file, in bytes.
    type: str = (
        ""  # Local file format, i.e., the file extension, such as txt. Format supports pdf, txt, doc, docx types.
    )
    status: DocumentStatus  # File processing status. Values include: 0: In processing, 1: Processing completed, 9: Processing failed, recommend re-uploading
    progress: int = 0  # File upload progress. Unit is percentage.
    update_type: DocumentUpdateType  # Will the webpage automatically update online. Values include:
    document_name: str = ""  # File name.
    remaining_time: int = 0  # Estimated remaining time, in seconds.
    status_descript: str = ""  # Detailed description of the failure status, for example, the failure information returned when slicing fails. This parameter is returned only when document processing fails.
    update_interval: int = 0  # Frequency of automatic updates for online web pages. Unit is hours.


class UpdateDatasetRes(CozeModel):
    pass


class DeleteDatasetRes(CozeModel):
    pass


class DatasetsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._documents: Optional[DatasetsDocumentsClient] = None
        self._images: Optional[DatasetsImagesClient] = None

    @property
    def documents(self) -> "DatasetsDocumentsClient":
        if self._documents is None:
            from .documents import DatasetsDocumentsClient

            self._documents = DatasetsDocumentsClient(base_url=self._base_url, requester=self._requester)
        return self._documents

    @property
    def images(self) -> "DatasetsImagesClient":
        if self._images is None:
            from .images import DatasetsImagesClient

            self._images = DatasetsImagesClient(base_url=self._base_url, requester=self._requester)
        return self._images

    def create(
        self,
        *,
        name: str,
        space_id: str,
        format_type: DocumentFormatType,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
    ) -> CreateDatasetResp:
        """
        Create Dataset

        docs en: https://www.coze.com/docs/developer_guides/create_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/create_dataset

        :param name: The name of the dataset
        :param space_id: The ID of the space that the dataset belongs to
        :param format_type: The format type of the dataset, 0: text, 2: image
        :param description: The description of the dataset
        :param icon_file_id: The ID of the icon file, uploaded by `coze.files.upload`
        """
        url = f"{self._base_url}/v1/datasets"
        body = {
            "name": name,
            "space_id": space_id,
            "format_type": format_type,
            "description": description,
            "file_id": icon_file_id,
        }
        return self._requester.request(
            "post",
            url,
            False,
            CreateDatasetResp,
            body=body,
        )

    def list(
        self,
        *,
        space_id: str,
        name: Optional[str] = None,
        format_type: Optional[DocumentFormatType] = None,
        page_num: int = 1,
        page_size: int = 10,
        **kwargs,
    ) -> NumberPaged[Dataset]:
        """
        List Datasets

        docs en: https://www.coze.com/docs/developer_guides/list_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/list_dataset

        :param space_id: The ID of the space that the dataset belongs to
        :param name: The name of the dataset, supports fuzzy search
        :param format_type: The format type of the dataset, 0: text, 1: table, 2: image
        :param page_num: The page number. The minimum value is 1, and the default is 1.
        :param page_size: The page size. The value range is 1~300, and the default is 10.
        """

        url = f"{self._base_url}/v1/datasets"
        headers: Optional[dict] = kwargs.get("headers")

        def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return self._requester.make_request(
                "GET",
                url,
                headers=headers,
                params={
                    "space_id": space_id,
                    "name": name,
                    "format_type": format_type,
                    "page_size": i_page_size,
                    "page_num": i_page_num,
                },
                cast=_PrivateListDatasetsData,
                stream=False,
            )

        return NumberPaged(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )

    def update(
        self,
        *,
        dataset_id: str,
        name: str,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
    ) -> UpdateDatasetRes:
        """
        Update Dataset
        This API will fully refresh the knowledge base's name, file_id, and description settings. If these parameters are not set, they will revert to default settings.

        docs en: https://www.coze.com/docs/developer_guides/update_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/update_dataset

        :param dataset_id: The ID of the dataset
        :param name: The name of the dataset
        :param description: The description of the dataset
        :param icon_file_id: The ID of the icon file, uploaded by `coze.files.upload`
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}"
        body = {
            "name": name,
            "description": description,
            "file_id": icon_file_id,
        }
        return self._requester.request(
            "put",
            url,
            False,
            cast=UpdateDatasetRes,
            body=body,
        )

    def delete(
        self,
        *,
        dataset_id: str,
    ) -> DeleteDatasetRes:
        """
        Delete Dataset
        The workspace administrator can delete all knowledge bases in the team, while other members can only delete knowledge bases they own.
        When deleting a knowledge base, all files uploaded to the knowledge base will be deleted simultaneously, and any agents bound to this knowledge base will be automatically unbound.

        docs en: https://www.coze.com/docs/developer_guides/delete_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/delete_dataset

        :param dataset_id: The ID of the dataset
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}"
        return self._requester.request(
            "delete",
            url,
            False,
            cast=DeleteDatasetRes,
        )

    def process(
        self,
        *,
        dataset_id: str,
        document_ids: List[str],
    ) -> ListResponse[DocumentProgress]:
        """
        Check the upload progress
        Call this API to get the upload progress of knowledge base files.
        This API supports viewing the upload progress of all types of knowledge base files, such as text, images, and tables.
        It supports batch viewing of multiple files' progress, but the files must be located in the same knowledge base.

        docs en: https://www.coze.com/docs/developer_guides/get_dataset_progress
        docs zh: https://www.coze.cn/docs/developer_guides/get_dataset_progress

        :param dataset_id: The ID of the dataset
        :param document_ids: The IDs of the documents
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/process"
        body = {
            "document_ids": document_ids,
        }
        return self._requester.request(
            "post",
            url,
            False,
            cast=ListResponse[DocumentProgress],
            body=body,
            data_field="data.data",
        )


class AsyncDatasetsClient(object):
    def __init__(self, base_url: str, requester: Requester):
        self._base_url = remove_url_trailing_slash(base_url)
        self._requester = requester
        self._documents: Optional[AsyncDatasetsDocumentsClient] = None
        self._images: Optional[AsyncDatasetsImagesClient] = None

    @property
    def documents(self) -> "AsyncDatasetsDocumentsClient":
        if self._documents is None:
            from .documents import AsyncDatasetsDocumentsClient

            self._documents = AsyncDatasetsDocumentsClient(base_url=self._base_url, requester=self._requester)
        return self._documents

    @property
    def images(self) -> "AsyncDatasetsImagesClient":
        if self._images is None:
            from .images import AsyncDatasetsImagesClient

            self._images = AsyncDatasetsImagesClient(base_url=self._base_url, requester=self._requester)
        return self._images

    async def create(
        self,
        *,
        name: str,
        space_id: str,
        format_type: DocumentFormatType,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
    ) -> CreateDatasetResp:
        """
        Create Dataset

        :param name: The name of the dataset
        :param space_id: The ID of the space that the dataset belongs to
        :param format_type: The format type of the dataset, 0: text, 2: image
        :param description: The description of the dataset
        :param icon_file_id: The ID of the icon file, uploaded by `coze.files.upload`
        """
        url = f"{self._base_url}/v1/datasets"
        body = {
            "name": name,
            "space_id": space_id,
            "format_type": format_type,
            "description": description,
            "file_id": icon_file_id,
        }
        return await self._requester.arequest(
            "post",
            url,
            False,
            CreateDatasetResp,
            body=body,
        )

    async def list(
        self,
        *,
        space_id: str,
        name: Optional[str] = None,
        format_type: Optional[DocumentFormatType] = None,
        page_num: int = 1,
        page_size: int = 10,
        **kwargs,
    ) -> AsyncNumberPaged[Dataset]:
        """
        List Datasets

        docs en: https://www.coze.com/docs/developer_guides/list_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/list_dataset

        :param space_id: The ID of the space that the dataset belongs to
        :param name: The name of the dataset, supports fuzzy search
        :param format_type: The format type of the dataset, 0: text, 1: table, 2: image
        :param page_num: The page number. The minimum value is 1, and the default is 1.
        :param page_size: The page size. The value range is 1~300, and the default is 10.
        """

        url = f"{self._base_url}/v1/datasets"
        headers: Optional[dict] = kwargs.get("headers")

        async def request_maker(i_page_num: int, i_page_size: int) -> HTTPRequest:
            return await self._requester.amake_request(
                "GET",
                url,
                headers=headers,
                params={
                    "space_id": space_id,
                    "name": name,
                    "format_type": format_type,
                    "page_size": i_page_size,
                    "page_num": i_page_num,
                },
                cast=_PrivateListDatasetsData,
                stream=False,
            )

        return await AsyncNumberPaged.build(
            page_num=page_num,
            page_size=page_size,
            requestor=self._requester,
            request_maker=request_maker,
        )

    async def update(
        self,
        *,
        dataset_id: str,
        name: str,
        description: Optional[str] = None,
        icon_file_id: Optional[str] = None,
    ) -> UpdateDatasetRes:
        """
        Update Dataset
        This API will fully refresh the knowledge base's name, file_id, and description settings. If these parameters are not set, they will revert to default settings.

        docs en: https://www.coze.com/docs/developer_guides/update_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/update_dataset

        :param dataset_id: The ID of the dataset
        :param name: The name of the dataset
        :param description: The description of the dataset
        :param icon_file_id: The ID of the icon file, uploaded by `coze.files.upload`
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}"
        body = {
            "name": name,
            "description": description,
            "file_id": icon_file_id,
        }
        return await self._requester.arequest(
            "put",
            url,
            False,
            cast=UpdateDatasetRes,
            body=body,
        )

    async def delete(
        self,
        *,
        dataset_id: str,
    ) -> DeleteDatasetRes:
        """
        Delete Dataset
        The workspace administrator can delete all knowledge bases in the team, while other members can only delete knowledge bases they own.
        When deleting a knowledge base, all files uploaded to the knowledge base will be deleted simultaneously, and any agents bound to this knowledge base will be automatically unbound.

        docs en: https://www.coze.com/docs/developer_guides/delete_dataset
        docs zh: https://www.coze.cn/docs/developer_guides/delete_dataset

        :param dataset_id: The ID of the dataset
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}"
        return await self._requester.arequest(
            "delete",
            url,
            False,
            cast=DeleteDatasetRes,
        )

    async def process(
        self,
        *,
        dataset_id: str,
        document_ids: List[str],
    ) -> ListResponse[DocumentProgress]:
        """
        Check the upload progress
        Call this API to get the upload progress of knowledge base files.
        This API supports viewing the upload progress of all types of knowledge base files, such as text, images, and tables.
        It supports batch viewing of multiple files' progress, but the files must be located in the same knowledge base.

        docs en: https://www.coze.com/docs/developer_guides/get_dataset_progress
        docs zh: https://www.coze.cn/docs/developer_guides/get_dataset_progress

        :param dataset_id: The ID of the dataset
        :param document_ids: The IDs of the documents
        """
        url = f"{self._base_url}/v1/datasets/{dataset_id}/process"
        body = {
            "document_ids": document_ids,
        }
        return await self._requester.arequest(
            "post",
            url,
            False,
            cast=ListResponse[DocumentProgress],
            body=body,
            data_field="data.data",
        )
